from six import python_2_unicode_compatible

from quickbooks.objects.detailline import DetailLine, ItemBasedExpenseLine, AccountBasedExpenseLine, \
    TDSLine
from .base import Ref, Address, QuickbooksManagedObject, LinkedTxnMixin, \
    QuickbooksTransactionEntity, CustomField, LinkedTxn
from .tax import TxnTaxDetail
from ..mixins import DeleteMixin, SendMixin


@python_2_unicode_compatible
class PurchaseOrder(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin, SendMixin):
    """
    QBO definition: The PurchaseOrder entity is a non-posting transaction representing a request to purchase
    goods or services from a third party.
    """
    class_dict = {
        "VendorAddr": Address,
        "ShipAddr": Address,
        "VendorRef": Ref,
        "APAccountRef": Ref,
        "AttachableRef": Ref,
        "ClassRef": Ref,
        "SalesTermRef": Ref,
        "ShipMethodRef": Ref,
        "TaxCodeRef": Ref,
        "CurrencyRef": Ref,
        "TxnTaxDetail": TxnTaxDetail
    }

    list_dict = {
        "Line": DetailLine,
        "CustomField": CustomField,
        "LinkedTxn": LinkedTxn,
    }

    detail_dict = {
        "ItemBasedExpenseLineDetail": ItemBasedExpenseLine,
        "AccountBasedExpenseLineDetail": AccountBasedExpenseLine,
        "TDSLineDetail": TDSLine,
    }

    qbo_object_name = "PurchaseOrder"

    def __init__(self):
        super(PurchaseOrder, self).__init__()
        self.POStatus = None
        self.DocNumber = None
        self.TxnDate = None
        self.PrivateNote = None
        self.TotalAmt = 0
        self.DueDate = None
        self.ExchangeRate = 1
        self.GlobalTaxCalculation = "TaxExcluded"
        self.Memo = None
        self.ShipMethodRef = None

        self.TxnTaxDetail = None
        self.VendorAddr = None
        self.ShipAddr = None
        self.VendorRef = None
        self.APAccountRef = None
        self.AttachableRef = None
        self.ClassRef = None
        self.SalesTermRef = None
        self.TaxCodeRef = None
        self.CurrencyRef = None
        self.TxnTaxDetail = None

        self.Line = []
        self.CustomField = []
        self.LinkedTxn = []

    def __str__(self):
        return str(self.TotalAmt)



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.vendorcredit import VendorCredit


class VendorCreditTests(unittest.TestCase):
    def test_unicode(self):
        vendor_credit = VendorCredit()
        vendor_credit.TotalAmt = 1000

        self.assertEquals(str(vendor_credit), "1000")

    def test_valid_object_name(self):
        obj = VendorCredit()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)




import unittest

from quickbooks import QuickBooks
from quickbooks.objects.vendor import Vendor, ContactInfo


class VendorTests(unittest.TestCase):
    def test_unicode(self):
        vendor = Vendor()
        vendor.DisplayName = "test"

        self.assertEquals(str(vendor), "test")

    def test_to_ref(self):
        vendor = Vendor()
        vendor.DisplayName = "test"
        vendor.Id = 100

        ref = vendor.to_ref()

        self.assertEquals(ref.name, "test")
        self.assertEquals(ref.type, "Vendor")
        self.assertEquals(ref.value, 100)

    def test_valid_object_name(self):
        obj = Vendor()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)


class ContactInfoTests(unittest.TestCase):
    def test_init(self):
        contact_info = ContactInfo()

        self.assertEquals(contact_info.Type, "")
        self.assertEquals(contact_info.Telephone, None)



import os
import tempfile
from datetime import datetime

from quickbooks.objects.attachable import Attachable
from quickbooks.objects.base import AttachableRef
from quickbooks.objects.vendor import Vendor
from tests.integration.test_base import QuickbooksTestCase


class AttachableTest(QuickbooksTestCase):
    def setUp(self):
        super(AttachableTest, self).setUp()
        self.time = datetime.now()

    def test_create_note(self):
        attachable = Attachable()

        vendor = Vendor.all(max_results=1, qb=self.qb_client)[0]

        attachable_ref = AttachableRef()
        attachable_ref.EntityRef = vendor.to_ref()
        attachable.AttachableRef.append(attachable_ref)

        attachable.Note = "Test note added on {}".format(self.time.strftime("%Y-%m-%d %H:%M:%S"))

        attachable.save(qb=self.qb_client)
        query_attachable = Attachable.get(attachable.Id, qb=self.qb_client)

        self.assertEquals(query_attachable.AttachableRef[0].EntityRef.value, vendor.Id)
        self.assertEquals(query_attachable.Note, "Test note added on {}".format(self.time.strftime("%Y-%m-%d %H:%M:%S")))

    def test_update_note(self):
        attachable = Attachable.all(max_results=1, qb=self.qb_client)[0]

        attachable.Note = "Note updated on {}".format(self.time.strftime("%Y-%m-%d %H:%M:%S"))
        attachable.save(qb=self.qb_client)

        query_attachable = Attachable.get(attachable.Id, qb=self.qb_client)
        self.assertEquals(query_attachable.Note, "Note updated on {}".format(self.time.strftime("%Y-%m-%d %H:%M:%S")))

    def test_create_file(self):
        attachable = Attachable()
        test_file = tempfile.NamedTemporaryFile(suffix=".txt")

        vendor = Vendor.all(max_results=1, qb=self.qb_client)[0]

        attachable_ref = AttachableRef()
        attachable_ref.EntityRef = vendor.to_ref()
        attachable.AttachableRef.append(attachable_ref)

        attachable.FileName = os.path.basename(test_file.name)
        attachable._FilePath = test_file.name
        attachable.ContentType = 'text/plain'

        attachable.save(qb=self.qb_client)

        query_attachable = Attachable.get(attachable.Id, qb=self.qb_client)

        self.assertEquals(query_attachable.AttachableRef[0].EntityRef.value, vendor.Id)



from six import python_2_unicode_compatible
from .base import Address, PhoneNumber, EmailAddress, WebAddress, Ref, QuickbooksBaseObject, \
    QuickbooksManagedObject, QuickbooksTransactionEntity


class ContactInfo(QuickbooksBaseObject):
    class_dict = {
        "Telephone": PhoneNumber
    }

    def __init__(self):
        super(ContactInfo, self).__init__()

        self.Type = ""
        self.Telephone = None


@python_2_unicode_compatible
class Vendor(QuickbooksManagedObject, QuickbooksTransactionEntity):
    """
    QBO definition: The Vendor represents the seller from whom your company purchases any service or product.
    """

    class_dict = {
        "BillAddr": Address,
        "TermRef": Ref,
        "PrimaryPhone": PhoneNumber,
        "AlternatePhone": PhoneNumber,
        "Mobile": PhoneNumber,
        "Fax": PhoneNumber,
        "PrimaryEmailAddr": EmailAddress,
        "WebAddr": WebAddress,
        "CurrencyRef": Ref,
        "APAccountRef": Ref
    }

    qbo_object_name = "Vendor"

    def __init__(self):
        super(Vendor, self).__init__()
        self.Title = ""
        self.GivenName = ""
        self.MiddleName = ""
        self.FamilyName = ""
        self.Suffix = ""
        self.CompanyName = ""
        self.DisplayName = ""
        self.PrintOnCheckName = ""
        self.Active = True
        self.TaxIdentifier = ""
        self.Balance = 0
        self.BillRate = 0
        self.AcctNum = ""
        self.Vendor1099 = True
        self.TaxReportingBasis = ""

        self.BillAddr = None
        self.PrimaryPhone = None
        self.AlternatePhone = None
        self.Mobile = None
        self.Fax = None
        self.PrimaryEmailAddr = None
        self.WebAddr = None
        self.TermRef = None
        self.CurrencyRef = None
        self.APAccountRef = None

    def __str__(self):
        return self.DisplayName

    def to_ref(self):
        ref = Ref()

        ref.name = self.DisplayName
        ref.type = self.qbo_object_name
        ref.value = self.Id

        return ref



import unittest

from quickbooks import QuickBooks
from quickbooks.objects import Ref
from quickbooks.objects.invoice import Invoice, DeliveryInfo


class InvoiceTests(unittest.TestCase):
    def test_unicode(self):
        invoice = Invoice()
        invoice.TotalAmt = 10

        self.assertEquals(str(invoice), "10")

    def test_to_LinkedTxn(self):
        invoice = Invoice()
        invoice.TotalAmt = 10
        invoice.Id = 1

        linked_txn = invoice.to_linked_txn()

        self.assertEquals(linked_txn.TxnId, invoice.Id)
        self.assertEquals(linked_txn.TxnType, "Invoice")
        self.assertEquals(linked_txn.TxnLineId, 1)

    def test_email_sent_true(self):
        invoice = Invoice()
        invoice.EmailStatus = "EmailSent"
        self.assertTrue(invoice.email_sent)

    def test_email_sent_false(self):
        invoice = Invoice()
        invoice.EmailStatus = "NotSent"
        self.assertFalse(invoice.email_sent)

    def test_valid_object_name(self):
        obj = Invoice()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)

    def test_to_ref(self):
        invoice = Invoice()
        invoice.DocNumber = 1
        invoice.Id = 2

        ref = invoice.to_ref()
        self.assertIsInstance(ref, Ref)
        self.assertEquals(ref.type, "Invoice")
        self.assertEquals(ref.name, 1)  # should be DocNumber
        self.assertEquals(ref.value, 2)  # should be Id


class DeliveryInfoTests(unittest.TestCase):
    def test_init(self):
        info = DeliveryInfo()

        self.assertEquals(info.DeliveryType, "")
        self.assertEquals(info.DeliveryTime, "")



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.company_info import CompanyInfo


class CompanyInfoTests(unittest.TestCase):
    def test_unicode(self):
        company_info = CompanyInfo()
        company_info.CompanyName = "test"

        self.assertEquals(str(company_info), "test")

    def test_to_ref(self):
        company_info = CompanyInfo()
        company_info.CompanyName = "test"
        company_info.Id = 100

        ref = company_info.to_ref()

        self.assertEquals(ref.name, "test")
        self.assertEquals(ref.type, "CompanyInfo")
        self.assertEquals(ref.value, 100)



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.timeactivity import TimeActivity


class TimeActivityTests(unittest.TestCase):
    def test_unicode(self):
        time_activity = TimeActivity()

        time_activity.NameOf = "test"
        time_activity.TimeZone = "CST"
        time_activity.BillableStatus = "test"
        time_activity.Taxable = False
        time_activity.HourlyRate = 0
        time_activity.Hours = 1
        time_activity.Minutes = 60
        time_activity.BreakHours = 1
        time_activity.BreakMinutes = 60
        time_activity.Description = "test"

        self.assertEquals(str(time_activity), "test")
        self.assertEquals(time_activity.TimeZone, "CST")
        self.assertEquals(time_activity.BillableStatus, "test")
        self.assertEquals(time_activity.Taxable, False)
        self.assertEquals(time_activity.HourlyRate, 0)
        self.assertEquals(time_activity.Hours, 1)
        self.assertEquals(time_activity.Minutes, 60)
        self.assertEquals(time_activity.BreakHours, 1)
        self.assertEquals(time_activity.BreakMinutes, 60)
        self.assertEquals(time_activity.Description, "test")

    def test_valid_object_name(self):
        obj = TimeActivity()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)



from six import python_2_unicode_compatible
from .base import Address, PhoneNumber, EmailAddress, WebAddress, MetaData, QuickbooksReadOnlyObject, \
    QuickbooksTransactionEntity


@python_2_unicode_compatible
class CustomerType(QuickbooksReadOnlyObject, QuickbooksTransactionEntity):
    """
    QBO definition: Customer types allow categorizing customers in ways that are meaningful to the business.
    For example, one could set up customer types so that they indicate which industry a customer represents,
    a customer's geographic location, or how a customer first heard about the business. The categorization
    then can be used for reporting or mailings.
    """

    class_dict = {
        "MetaData": MetaData
    }

    qbo_object_name = "CustomerType"

    def __init__(self):
        super(CustomerType, self).__init__()
        self.Name = ""
        self.Active = False
        self.MetaData = None

    def __str__(self):
        return self.Name



from datetime import datetime

from quickbooks.objects.companycurrency import CompanyCurrency
from tests.integration.test_base import QuickbooksUnitTestCase


class CompanyCurrencyTest(QuickbooksUnitTestCase):
    def test_unicode(self):
        company_currency = CompanyCurrency()
        company_currency.Name = "test"
        company_currency.Code = "USD"

        self.assertEquals(str(company_currency), "test")

    def test_to_ref(self):
        company_currency = CompanyCurrency()
        company_currency.Name = "test"
        company_currency.Id = 23

        ref = company_currency.to_ref()

        self.assertEquals(ref.name, "test")
        self.assertEquals(ref.type, "CompanyCurrency")
        self.assertEquals(ref.value, 23)


import time
from datetime import datetime

from quickbooks.objects import Transfer
from quickbooks.objects.account import Account
from quickbooks.objects.creditcardpayment_entity import CreditCardPayment
from tests.integration.test_base import QuickbooksTestCase


class CreditCardPaymentEntityTest(QuickbooksTestCase):
    def setUp(self):
        time.sleep(3)  # Used to prevent error code 3001 - The request limit was reached.
        super(CreditCardPaymentEntityTest, self).setUp()

        self.account_number = datetime.now().strftime('%d%H%M')
        self.name = "Test CreditCardPaymentEntityTest {0}".format(self.account_number)

    def test_create(self):
        credit_card_account = Account()
        credit_card_account.Name = "Credit Card Account {0}".format(self.account_number)
        credit_card_account.AccountType = "Credit Card"
        credit_card_account.AccountSubType = "CreditCard"
        credit_card_account.save(qb=self.qb_client)

        accounts = Account.where(
            "Classification = 'Asset' AND FullyQualifiedName != 'Accounts Receivable (A/R)'",
            max_results=1, qb=self.qb_client)

        from_account = accounts[0]
        to_account = credit_card_account

        credit_card_payment = CreditCardPayment()
        credit_card_payment.Amount = 100
        credit_card_payment.BankAccountRef = from_account.to_ref()
        credit_card_payment.CreditCardAccountRef = to_account.to_ref()

        credit_card_payment.save(qb=self.qb_client)

        query_credit_card_payment = CreditCardPayment.get(credit_card_payment.Id, qb=self.qb_client)

        self.assertEquals(query_credit_card_payment.Id, credit_card_payment.Id)
        self.assertEquals(query_credit_card_payment.Amount, 100)
        self.assertEquals(query_credit_card_payment.BankAccountRef.value, from_account.Id)
        self.assertEquals(query_credit_card_payment.CreditCardAccountRef.value, to_account.Id)

        # reset transfer (so the from_account doesn't run out of cash)
        # I wonder if we can do a transfer from credit_card_account to a bank_account
        transfer = Transfer()
        transfer.Amount = 100
        transfer.FromAccountRef = to_account.to_ref()
        transfer.ToAccountRef = from_account.to_ref()

        transfer.save(qb=self.qb_client)

    def test_update(self):
        credit_card_payment = CreditCardPayment.all(max_results=1, qb=self.qb_client)[0]
        credit_card_payment.Amount += 1
        credit_card_payment.save(qb=self.qb_client)

        query_credit_card_payment = CreditCardPayment.get(credit_card_payment.Id, qb=self.qb_client)

        self.assertEquals(query_credit_card_payment.Amount, credit_card_payment.Amount)



from six import python_2_unicode_compatible
from .base import QuickbooksBaseObject, Ref, LinkedTxn, QuickbooksManagedObject, LinkedTxnMixin, \
    QuickbooksTransactionEntity
from ..mixins import DeleteMixin


@python_2_unicode_compatible
class CheckPayment(QuickbooksBaseObject):
    class_dict = {
        "BankAccountRef": Ref
    }

    qbo_object_name = "CheckPayment"

    def __init__(self):
        super(CheckPayment, self).__init__()
        self.PrintStatus = "NotSet"
        self.BankAccountRef = None

    def __str__(self):
        return self.PrintStatus


class BillPaymentCreditCard(QuickbooksBaseObject):
    class_dict = {
        "CCAccountRef": Ref
    }

    qbo_object_name = "BillPaymentCreditCard"

    def __init__(self):
        super(BillPaymentCreditCard, self).__init__()
        self.CCAccountRef = None


@python_2_unicode_compatible
class BillPaymentLine(QuickbooksBaseObject):
    list_dict = {
        "LinkedTxn": LinkedTxn
    }

    qbo_object_name = "Line"

    def __init__(self):
        super(BillPaymentLine, self).__init__()
        self.Amount = 0
        self.LinkedTxn = []

    def __str__(self):
        return str(self.Amount)


@python_2_unicode_compatible
class BillPayment(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin):
    """
    QBO definition: A BillPayment entity represents the financial transaction of payment
    of bills that the business owner receives from a vendor for goods or services purchased
    from the vendor. QuickBooks Online supports bill payments through a credit card or a
    checking account. BillPayment.TotalAmt is the total amount associated with this payment.
    This includes the total of all the payments from the payment line details. If TotalAmt is
    greater than the total on the lines being paid, the overpayment is treated as a credit and
    exposed as such on the QuickBooks UI. The total amount cannot be negative.
    """

    class_dict = {
        "VendorRef": Ref,
        "CheckPayment": CheckPayment,
        "CreditCardPayment": BillPaymentCreditCard,
        "APAccountRef": Ref,
        "DepartmentRef": Ref,
        "CurrencyRef": Ref
    }

    list_dict = {
        "Line": BillPaymentLine
    }

    qbo_object_name = "BillPayment"

    def __init__(self):
        super(BillPayment, self).__init__()
        self.PayType = ""
        self.TotalAmt = 0
        self.PrivateNote = ""
        self.DocNumber = ""

        self.VendorRef = None
        self.CheckPayment = None
        self.APAccountRef = None
        self.DepartmentRef = None
        self.CreditCardPayment = None
        self.CurrencyRef = None

        self.Line = []

    def __str__(self):
        return str(self.TotalAmt)



from six import python_2_unicode_compatible
from .base import QuickbooksBaseObject, Ref, QuickbooksManagedObject, QuickbooksTransactionEntity, \
    LinkedTxnMixin
from .tax import TxnTaxDetail
from .detailline import DetailLine, DescriptionOnlyLine
from ..mixins import DeleteMixin


class Entity(QuickbooksBaseObject):
    class_dict = {
        "EntityRef": Ref
    }

    def __init__(self):
        super(Entity, self).__init__()
        self.Type = ""
        self.EntityRef = None


class JournalEntryLineDetail(QuickbooksBaseObject):
    class_dict = {
        "Entity": Entity,
        "AccountRef": Ref,
        "ClassRef": Ref,
        "DepartmentRef": Ref,
        "TaxCodeRef": Ref,
    }

    def __init__(self):
        super(JournalEntryLineDetail, self).__init__()
        self.PostingType = ""
        self.TaxApplicableOn = "Sales"
        self.TaxAmount = 0

        self.BillableStatus = None
        self.Entity = None
        self.AccountRef = None
        self.ClassRef = None
        self.DepartmentRef = None
        self.TaxCodeRef = None


class JournalEntryLine(DetailLine):
    class_dict = {
        "JournalEntryLineDetail": JournalEntryLineDetail
    }

    def __init__(self):
        super(JournalEntryLine, self).__init__()
        self.DetailType = "JournalEntryLineDetail"
        self.JournalEntryLineDetail = None


@python_2_unicode_compatible
class JournalEntry(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin):
    """
    QBO definition: Journal Entry is a transaction in which:
        - There are at least two parts - a Debit and a Credit - called distribution lines.
        - Each distribution line has an account from the Chart of Accounts.
        - The total of the Debit column equals the total of the Credit column.

        When you record a transaction with Journal Entry, the QBO UI labels the transaction as JRNL in a
        register and General Journal on reports that list transactions.
    """

    class_dict = {
        "TxnTaxDetail": TxnTaxDetail,
        "CurrencyRef": Ref,
    }

    list_dict = {
        "Line": DetailLine
    }

    detail_dict = {
        "DescriptionOnly": DescriptionOnlyLine,
        "JournalEntryLineDetail": JournalEntryLine
    }

    qbo_object_name = "JournalEntry"

    def __init__(self):
        super(JournalEntry, self).__init__()
        self.Adjustment = False
        self.TxnDate = ""
        #self.TxnSource = ""
        self.DocNumber = ""
        self.PrivateNote = ""
        self.TotalAmt = 0
        self.ExchangeRate = 1
        self.Line = []
        self.TxnTaxDetail = None

        self.CurrencyRef = None

    def __str__(self):
        return str(self.TotalAmt)



import unittest

from quickbooks.objects.creditcardpayment import CreditCardPayment, CreditChargeResponse, CreditChargeInfo


class CreditCardPaymentTests(unittest.TestCase):
    def test_init(self):
        payment = CreditCardPayment()

        self.assertEquals(payment.CreditChargeInfo, None)
        self.assertEquals(payment.CreditChargeResponse, None)


class CreditChargeResponseTests(unittest.TestCase):
    def test_init(self):
        response = CreditChargeResponse()

        self.assertEquals(response.CCTransId, "")
        self.assertEquals(response.AuthCode, "")
        self.assertEquals(response.TxnAuthorizationTime, "")
        self.assertEquals(response.Status, "")


class CreditChargeInfoTests(unittest.TestCase):
    def test_init(self):
        info = CreditChargeInfo()

        self.assertEquals(info.Type, "")
        self.assertEquals(info.NameOnAcct, "")
        self.assertEquals(info.CcExpiryMonth, 0)
        self.assertEquals(info.CcExpiryYear, 0)
        self.assertEquals(info.BillAddrStreet, "")
        self.assertEquals(info.PostalCode, "")
        self.assertEquals(info.Amount, 0)
        self.assertEquals(info.ProcessPayment, False)



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.taxcode import TaxCode, TaxRateDetail, TaxRateList


class TaxCodeTests(unittest.TestCase):
    def test_unicode(self):
        taxcode = TaxCode()
        taxcode.Name = "test"

        self.assertEquals(str(taxcode), "test")

    def test_valid_object_name(self):
        obj = TaxCode()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)

    def test_to_ref(self):
        taxcode = TaxCode()
        taxcode.Id = 2
        taxcode.Name = "test"

        ref = taxcode.to_ref()
        self.assertEquals(ref.name, "test")
        self.assertEquals(ref.type, "TaxCode")
        self.assertEquals(ref.value, 2)


class TaxRateDetailTests(unittest.TestCase):
    def test_init(self):
        tax_rate = TaxRateDetail()

        self.assertEquals(tax_rate.TaxOrder, 0)
        self.assertEquals(tax_rate.TaxTypeApplicable, "")


class TaxRateListTests(unittest.TestCase):
    def test_init(self):
        tax_rate_list = TaxRateList()

        self.assertEquals(tax_rate_list.TaxRateDetail, [])




import os

import unittest
from future.moves.urllib.parse import quote

from quickbooks.objects import Bill, Invoice

from tests.integration.test_base import QuickbooksUnitTestCase

try:
    from mock import patch
except ImportError:
    from unittest.mock import patch

from quickbooks import client

from quickbooks.objects.base import PhoneNumber, QuickbooksBaseObject
from quickbooks.objects.department import Department
from quickbooks.objects.customer import Customer
from quickbooks.objects.journalentry import JournalEntry, JournalEntryLine
from quickbooks.objects.salesreceipt import SalesReceipt
from quickbooks.mixins import ObjectListMixin


class ToJsonMixinTest(unittest.TestCase):
    def test_to_json(self):
        phone = PhoneNumber()
        phone.FreeFormNumber = "555-555-5555"

        json = phone.to_json()

        self.assertEquals(json, '{\n    "FreeFormNumber": "555-555-5555"\n}')


class FromJsonMixinTest(unittest.TestCase):
    def setUp(self):
        self.json_data = {
            'DocNumber': '123',
            'TotalAmt': 100,
            'Line': [
                {
                    "Id": "0",
                    "Description": "Test",
                    "Amount": 25.54,
                    "DetailType": "JournalEntryLineDetail",
                    "JournalEntryLineDetail": {
                        "PostingType": "Debit",
                    }
                },
            ],
        }

    def test_from_json(self):
        entry = JournalEntry()
        new_obj = entry.from_json(self.json_data)

        self.assertEquals(type(new_obj), JournalEntry)
        self.assertEquals(new_obj.DocNumber, "123")
        self.assertEquals(new_obj.TotalAmt, 100)

        line = new_obj.Line[0]
        self.assertEquals(type(line), JournalEntryLine)
        self.assertEquals(line.Description, "Test")
        self.assertEquals(line.Amount, 25.54)
        self.assertEquals(line.DetailType, "JournalEntryLineDetail")
        self.assertEquals(line.JournalEntryLineDetail.PostingType, "Debit")

    def test_from_json_missing_detail_object(self):
        test_obj = QuickbooksBaseObject()

        new_obj = test_obj.from_json(self.json_data)

        self.assertEquals(type(new_obj), QuickbooksBaseObject)
        self.assertEquals(new_obj.DocNumber, "123")
        self.assertEquals(new_obj.TotalAmt, 100)


class ToDictMixinTest(unittest.TestCase):
    def test_to_dict(self):
        json_data = {
            'DocNumber': '123',
            'TotalAmt': 100,
            'Line': [
                {
                    "Id": "0",
                    "Description": "Test",
                    "Amount": 25.54,
                    "DetailType": "JournalEntryLineDetail",
                    "JournalEntryLineDetail": {
                        "PostingType": "Debit",
                    }
                },
            ],
        }

        entry = JournalEntry.from_json(json_data)
        expected = {
            'DocNumber': '123',
            'SyncToken': 0,
            'domain': 'QBO',
            'TxnDate': '',
            'TotalAmt': 100,
            'ExchangeRate': 1,
            'CurrencyRef': None,
            'PrivateNote': '',
            'sparse': False,
            'Line': [{
                'LinkedTxn': [],
                'Description': 'Test',
                'JournalEntryLineDetail': {
                    'TaxAmount': 0,
                    'Entity': None,
                    'DepartmentRef': None,
                    'TaxCodeRef': None,
                    'BillableStatus': None,
                    'TaxApplicableOn': 'Sales',
                    'PostingType': 'Debit',
                    'AccountRef': None,
                    'ClassRef': None,
                },
                'DetailType': 'JournalEntryLineDetail',
                'LineNum': 0,
                'Amount': 25.54,
                'CustomField': [],
                'Id': '0',
            }],
            'Adjustment': False,
            'Id': None,
            'TxnTaxDetail': None,
        }

        self.assertEquals(expected, entry.to_dict())


class ListMixinTest(QuickbooksUnitTestCase):
    @patch('quickbooks.mixins.ListMixin.where')
    def test_all(self, where):
        Department.all()
        where.assert_called_once_with('', order_by='', max_results=100, start_position='', qb=None)

    def test_all_with_qb(self):
        with patch.object(self.qb_client, 'query') as query:
            Department.all(qb=self.qb_client)
            self.assertTrue(query.called)

    @patch('quickbooks.mixins.ListMixin.where')
    def test_filter(self, where):
        Department.filter(max_results=25, start_position='1', Active=True)
        where.assert_called_once_with("Active = True", max_results=25, start_position='1',
                                      order_by='', qb=None)

    def test_filter_with_qb(self):
        with patch.object(self.qb_client, 'query') as query:
            Department.filter(Active=True, qb=self.qb_client)
            self.assertTrue(query.called)

    @patch('quickbooks.mixins.ListMixin.query')
    def test_where(self, query):
        Department.where("Active=True", start_position=1, max_results=10)
        query.assert_called_once_with("SELECT * FROM Department WHERE Active=True STARTPOSITION 1 MAXRESULTS 10",
                                      qb=None)

    @patch('quickbooks.mixins.ListMixin.query')
    def test_where_start_position_0(self, query):
        Department.where("Active=True", start_position=0, max_results=10)
        query.assert_called_once_with("SELECT * FROM Department WHERE Active=True STARTPOSITION 0 MAXRESULTS 10",
                                      qb=None)

    def test_where_with_qb(self):
        with patch.object(self.qb_client, 'query') as query:
            Department.where("Active=True", start_position=1, max_results=10, qb=self.qb_client)
            self.assertTrue(query.called)

    @patch('quickbooks.mixins.QuickBooks.query')
    def test_query(self, query):
        select = "SELECT * FROM Department WHERE Active=True"
        Department.query(select)
        query.assert_called_once_with(select)

    def test_query_with_qb(self):
        with patch.object(self.qb_client, 'query') as query:
            select = "SELECT * FROM Department WHERE Active=True"
            Department.query(select, qb=self.qb_client)
            self.assertTrue(query.called)

    @patch('quickbooks.mixins.ListMixin.where')
    def test_choose(self, where):
        Department.choose(['name1', 'name2'], field="Name")
        where.assert_called_once_with("Name in ('name1', 'name2')", qb=None)

    def test_choose_with_qb(self):
        with patch.object(self.qb_client, 'query') as query:
            Department.choose(['name1', 'name2'], field="Name", qb=self.qb_client)
            self.assertTrue(query.called)

    @patch('quickbooks.mixins.QuickBooks.query')
    def test_count(self, query):
        count = Department.count(where_clause="Active=True", qb=self.qb_client)
        query.assert_called_once_with("SELECT COUNT(*) FROM Department WHERE Active=True")

    @patch('quickbooks.mixins.ListMixin.query')
    def test_order_by(self, query):
        Customer.filter(Active=True, order_by='DisplayName')
        query.assert_called_once_with("SELECT * FROM Customer WHERE Active = True ORDERBY DisplayName", qb=None)

    def test_order_by_with_qb(self):
        with patch.object(self.qb_client, 'query') as query:
            Customer.filter(Active=True, order_by='DisplayName', qb=self.qb_client)
            self.assertTrue(query.called)


class ReadMixinTest(QuickbooksUnitTestCase):
    @patch('quickbooks.mixins.QuickBooks.get_single_object')
    def test_get(self, get_single_object):
        Department.get(1)
        get_single_object.assert_called_once_with("Department", pk=1)

    def test_get_with_qb(self):
        with patch.object(self.qb_client, 'get_single_object') as get_single_object:
            Department.get(1, qb=self.qb_client)
            self.assertTrue(get_single_object.called)


class UpdateMixinTest(QuickbooksUnitTestCase):
    @patch('quickbooks.mixins.QuickBooks.create_object')
    def test_save_create(self, create_object):
        department = Department()
        department.save(qb=self.qb_client)
        create_object.assert_called_once_with("Department", department.to_json(), request_id=None)

    def test_save_create_with_qb(self):
        with patch.object(self.qb_client, 'create_object') as create_object:
            department = Department()
            department.save(qb=self.qb_client)
            self.assertTrue(create_object.called)

    @patch('quickbooks.mixins.QuickBooks.update_object')
    def test_save_update(self, update_object):
        department = Department()
        department.Id = 1
        json = department.to_json()

        department.save(qb=self.qb_client)
        update_object.assert_called_once_with("Department", json, request_id=None)

    def test_save_update_with_qb(self):
        with patch.object(self.qb_client, 'update_object') as update_object:
            department = Department()
            department.Id = 1
            json = department.to_json()

            department.save(qb=self.qb_client)
            self.assertTrue(update_object.called)


class DownloadPdfTest(QuickbooksUnitTestCase):
    @patch('quickbooks.client.QuickBooks.download_pdf')
    def test_download_invoice(self, download_pdf):
        receipt = SalesReceipt()
        receipt.Id = "1"

        receipt.download_pdf(self.qb_client)
        download_pdf.assert_called_once_with('SalesReceipt', "1")

    def test_download_missing_id(self):
        from quickbooks.exceptions import QuickbooksException

        receipt = SalesReceipt()
        self.assertRaises(QuickbooksException, receipt.download_pdf)


class ObjectListTest(unittest.TestCase):

    def setUp(self):
        class TestSubclass(ObjectListMixin):

            def __init__(self, obj_list):
                super(TestSubclass, self).__init__()
                self._object_list = obj_list

        self.TestSubclass = TestSubclass

    def test_object_list_mixin_with_primitives(self):

        test_primitive_list = [1, 2, 3]
        test_subclass_primitive_obj = self.TestSubclass(test_primitive_list)
        self.assertEquals(test_primitive_list, test_subclass_primitive_obj[:])

        for index in range(0, len(test_subclass_primitive_obj)):
            self.assertEquals(test_primitive_list[index], test_subclass_primitive_obj[index])

        for prim in test_subclass_primitive_obj:
            self.assertEquals(True, prim in test_subclass_primitive_obj)

        self.assertEquals(3, test_subclass_primitive_obj.pop())
        test_subclass_primitive_obj.append(4)
        self.assertEquals([1, 2, 4], test_subclass_primitive_obj[:])

        test_subclass_primitive_obj[0] = 5
        self.assertEquals([5, 2, 4], test_subclass_primitive_obj[:])

        del test_subclass_primitive_obj[0]
        self.assertEquals([2, 4], test_subclass_primitive_obj[:])

        self.assertEquals([4, 2], list(reversed(test_subclass_primitive_obj)))

    def test_object_list_mixin_with_qb_objects(self):

        pn1, pn2, pn3, pn4, pn5 = PhoneNumber(), PhoneNumber(), PhoneNumber(), PhoneNumber(), PhoneNumber()
        test_object_list = [pn1, pn2, pn3]
        test_subclass_object_obj = self.TestSubclass(test_object_list)
        self.assertEquals(test_object_list, test_subclass_object_obj[:])

        for index in range (0, len(test_subclass_object_obj)):
            self.assertEquals(test_object_list[index], test_subclass_object_obj[index])

        for obj in test_subclass_object_obj:
            self.assertEquals(True, obj in test_subclass_object_obj)

        self.assertEquals(pn3, test_subclass_object_obj.pop())
        test_subclass_object_obj.append(pn4)
        self.assertEquals([pn1, pn2, pn4], test_subclass_object_obj[:])

        test_subclass_object_obj[0] = pn5
        self.assertEquals([pn5, pn2, pn4], test_subclass_object_obj[:])

        del test_subclass_object_obj[0]
        self.assertEquals([pn2, pn4], test_subclass_object_obj[:])

        self.assertEquals([pn4, pn2], list(reversed(test_subclass_object_obj)))


class DeleteMixinTest(QuickbooksUnitTestCase):
    def test_delete_unsaved_exception(self):
        from quickbooks.exceptions import QuickbooksException

        bill = Bill()
        self.assertRaises(QuickbooksException, bill.delete, qb=self.qb_client)

    @patch('quickbooks.mixins.QuickBooks.delete_object')
    def test_delete(self, delete_object):
        bill = Bill()
        bill.Id = 1
        bill.delete(qb=self.qb_client)

        self.assertTrue(delete_object.called)


class SendMixinTest(QuickbooksUnitTestCase):
    @patch('quickbooks.mixins.QuickBooks.misc_operation')
    def test_send(self, mock_misc_op):
        invoice = Invoice()
        invoice.Id = 2
        invoice.send(qb=self.qb_client)

        mock_misc_op.assert_called_with("invoice/2/send", None, 'application/octet-stream')

    @patch('quickbooks.mixins.QuickBooks.misc_operation')
    def test_send_with_send_to_email(self, mock_misc_op):
        invoice = Invoice()
        invoice.Id = 2
        email = "test@email.com"
        send_to_email = quote(email, safe='')

        invoice.send(qb=self.qb_client, send_to=email)

        mock_misc_op.assert_called_with("invoice/2/send?sendTo={}".format(send_to_email), None, 'application/octet-stream')


class VoidMixinTest(QuickbooksUnitTestCase):
    @patch('quickbooks.mixins.QuickBooks.post')
    def test_void(self, post):
        invoice = Invoice()
        invoice.Id = 2
        invoice.void(qb=self.qb_client)
        self.assertTrue(post.called)

    def test_delete_unsaved_exception(self):
        from quickbooks.exceptions import QuickbooksException

        invoice = Invoice()
        self.assertRaises(QuickbooksException, invoice.void, qb=self.qb_client)



from six import python_2_unicode_compatible
from .base import QuickbooksManagedObject, QuickbooksTransactionEntity, Ref


@python_2_unicode_compatible
class Department(QuickbooksManagedObject, QuickbooksTransactionEntity):
    """
    QBO definition: The Department entity provides a way to track different segments of the business, divisions, or
    physical locations such as stores, and allows another way of categorizing the entire transaction.
    This is in contrast to Class objects, which are applied to individual transaction line details.
    Delete is achieved by setting the Active attribute to false in an entity update request; thus,
    making it inactive. In this type of delete, the record is not permanently deleted, but is hidden
    for display purposes. References to inactive objects are left intact.
    """

    class_dict = {
        "ParentRef": Ref
    }

    qbo_object_name = "Department"

    def __init__(self):
        super(Department, self).__init__()
        self.Name = ""
        self.SubDepartment = False
        self.FullyQualifiedName = ""
        self.Active = True

    def __str__(self):
        return self.Name

    def to_ref(self):
        ref = Ref()

        ref.name = self.Name
        ref.type = self.qbo_object_name
        ref.value = self.Id

        return ref



from six import python_2_unicode_compatible

from quickbooks.objects.detailline import DetailLine, ItemBasedExpenseLine, AccountBasedExpenseLine, \
    TDSLine
from .base import Ref, LinkedTxn, QuickbooksManagedObject, QuickbooksTransactionEntity, \
    LinkedTxnMixin
from .tax import TxnTaxDetail
from ..mixins import DeleteMixin


@python_2_unicode_compatible
class Bill(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin):
    """
    QBO definition: A Bill entity is an AP transaction representing a request-for-payment from a third party for
    goods/services rendered and/or received.
    """

    class_dict = {
        "SalesTermRef": Ref,
        "CurrencyRef": Ref,
        "APAccountRef": Ref,
        "VendorRef": Ref,
        "AttachableRef": Ref,
        "DepartmentRef": Ref,
        "TxnTaxDetail": TxnTaxDetail,
    }

    list_dict = {
        "Line": DetailLine,
        "LinkedTxn": LinkedTxn,
    }

    detail_dict = {
        "ItemBasedExpenseLineDetail": ItemBasedExpenseLine,
        "AccountBasedExpenseLineDetail": AccountBasedExpenseLine,
        "TDSLineDetail": TDSLine,
    }

    qbo_object_name = "Bill"

    def __init__(self):
        super(Bill, self).__init__()

        self.DueDate = ""
        self.Balance = 0
        self.TotalAmt = ""
        self.TxnDate = ""
        self.DocNumber = ""
        self.PrivateNote = ""
        self.ExchangeRate = 0
        self.GlobalTaxCalculation = None

        self.SalesTermRef = None
        self.CurrencyRef = None
        self.AttachableRef = None
        self.VendorRef = None
        self.DepartmentRef = None
        self.APAccountRef = None

        self.LinkedTxn = []
        self.Line = []

    def __str__(self):
        return str(self.Balance)

    def to_linked_txn(self):
        linked_txn = LinkedTxn()
        linked_txn.TxnId = self.Id
        linked_txn.TxnType = "Bill"
        linked_txn.TxnLineId = 1

        return linked_txn

    def to_ref(self):
        ref = Ref()

        ref.name = self.DocNumber
        ref.type = self.qbo_object_name
        ref.value = self.Id

        return ref




import unittest

from quickbooks import QuickBooks
from quickbooks.objects.purchase import Purchase


class PurchaseTests(unittest.TestCase):
    def test_unicode(self):
        purchase = Purchase()
        purchase.TotalAmt = 1000

        self.assertEquals(str(purchase), "1000")

    def test_valid_object_name(self):
        obj = Purchase()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)


import unittest

from quickbooks import QuickBooks
from quickbooks.objects.paymentmethod import PaymentMethod


class PaymentMethodTests(unittest.TestCase):
    def test_unicode(self):
        payment_method = PaymentMethod()
        payment_method.Name = "test"

        self.assertEquals(str(payment_method), "test")

    def test_valid_object_name(self):
        obj = PaymentMethod()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)

    def test_to_ref(self):
        obj = PaymentMethod()
        obj.Name = "test"
        obj.Id = 12

        ref = obj.to_ref()

        self.assertEquals(ref.name, "test")
        self.assertEquals(ref.type, "PaymentMethod")
        self.assertEquals(ref.value, 12)



from datetime import datetime

from quickbooks.objects import Customer, PaymentMethod
from quickbooks.objects.account import Account
from quickbooks.objects.payment import Payment

from tests.integration.test_base import QuickbooksTestCase


class PaymentTest(QuickbooksTestCase):
    def setUp(self):
        super(PaymentTest, self).setUp()

        self.account_number = datetime.now().strftime('%d%H%M')
        self.name = "Test Account {0}".format(self.account_number)

    def test_getall(self):
        payments = Payment.all(max_results=5, qb=self.qb_client)

        self.assertEqual(len(payments), 5)
        self.assertNotEqual(payments[0].Id, 0)

    def test_create(self):
        payment = Payment()
        payment.TotalAmt = 140.0

        customer = Customer.all(max_results=1, qb=self.qb_client)[0]
        payment.CustomerRef = customer.to_ref()

        payment_method = PaymentMethod.all(max_results=1, qb=self.qb_client)[0]

        payment.PaymentMethodRef = payment_method.to_ref()
        payment.save(qb=self.qb_client)

        query_payment = Payment.get(payment.Id, qb=self.qb_client)

        self.assertEqual(query_payment.CustomerRef.name, customer.DisplayName)
        self.assertEqual(query_payment.TotalAmt, 140.0)
        self.assertEqual(query_payment.PaymentMethodRef.value, payment_method.Id)

    def test_void(self):
        payment = Payment()
        payment.TotalAmt = 100.0

        customer = Customer.all(max_results=1, qb=self.qb_client)[0]
        payment.CustomerRef = customer.to_ref()

        payment_method = PaymentMethod.all(max_results=1, qb=self.qb_client)[0]

        payment.PaymentMethodRef = payment_method.to_ref()
        payment.save(qb=self.qb_client)

        query_payment = Payment.get(payment.Id, qb=self.qb_client)
        self.assertEqual(query_payment.TotalAmt, 100.0)

        payment.void(qb=self.qb_client)
        query_payment = Payment.get(payment.Id, qb=self.qb_client)

        self.assertEqual(query_payment.TotalAmt, 0.0)
        self.assertIn('Voided', query_payment.PrivateNote)



from quickbooks.objects.base import CustomerMemo
from quickbooks.objects.customer import Customer
from quickbooks.objects.detailline import SalesItemLine, SalesItemLineDetail
from quickbooks.objects.invoice import Invoice
from quickbooks.objects.item import Item
from tests.integration.test_base import QuickbooksTestCase
import uuid

class InvoiceTest(QuickbooksTestCase):
    def create_invoice(self, customer, request_id=None):
        invoice = Invoice()

        line = SalesItemLine()
        line.LineNum = 1
        line.Description = "description"
        line.Amount = 100
        line.SalesItemLineDetail = SalesItemLineDetail()
        item = Item.all(max_results=1, qb=self.qb_client)[0]

        line.SalesItemLineDetail.ItemRef = item.to_ref()
        invoice.Line.append(line)

        invoice.CustomerRef = customer.to_ref()

        invoice.CustomerMemo = CustomerMemo()
        invoice.CustomerMemo.value = "Customer Memo"
        invoice.save(qb=self.qb_client, request_id=request_id)
        return invoice
      
    def test_query_by_customer_ref(self):
        customer = Customer.all(max_results=1, qb=self.qb_client)[0]
        invoice = Invoice.query(
            "select * from Invoice where CustomerRef = '{0}'".format(customer.Id), qb=self.qb_client)

        print(invoice[0].Line[0].LineNum)
        print(invoice[0].Line[0].Amount)
        self.assertEquals(invoice[0].CustomerRef.name, customer.DisplayName)

    def test_where(self):
        customer = Customer.all(max_results=1, qb=self.qb_client)[0]

        invoice = Invoice.where(
            "CustomerRef = '{0}'".format(customer.Id), qb=self.qb_client)

        print(invoice[0])
        self.assertEquals(invoice[0].CustomerRef.name, customer.DisplayName)

    def test_create(self):
        customer = Customer.all(max_results=1, qb=self.qb_client)[0]
        invoice = self.create_invoice(customer)
        query_invoice = Invoice.get(invoice.Id, qb=self.qb_client)

        self.assertEquals(query_invoice.CustomerRef.name, customer.DisplayName)
        self.assertEquals(query_invoice.CustomerMemo.value, "Customer Memo")
        self.assertEquals(query_invoice.Line[0].Description, "description")
        self.assertEquals(query_invoice.Line[0].Amount, 100.0)
    
    def test_create_idempotence(self):
        customer = Customer.all(max_results=1, qb=self.qb_client)[0]
        sample_request_id = str(uuid.uuid4())
        invoice = self.create_invoice(customer, request_id=sample_request_id)
        duplicate_invoice = self.create_invoice(customer, request_id=sample_request_id)

        # Assert that both returned invoices have the same id
        self.assertEquals(invoice.Id, duplicate_invoice.Id)

    def test_delete(self):
        customer = Customer.all(max_results=1, qb=self.qb_client)[0]
        # First create an invoice
        invoice = self.create_invoice(customer)

        # Then delete
        invoice_id = invoice.Id
        invoice.delete(qb=self.qb_client)

        query_invoice = Invoice.filter(Id=invoice_id, qb=self.qb_client)
        self.assertEqual([], query_invoice)



from datetime import datetime

from quickbooks.helpers import qb_datetime_utc_offset_format
from quickbooks.objects.employee import Employee
from quickbooks.objects.timeactivity import TimeActivity
from tests.integration.test_base import QuickbooksTestCase


class TimeActivityTest(QuickbooksTestCase):
    def setUp(self):
        super(TimeActivityTest, self).setUp()

        self.name = "Test {0}".format(datetime.now().strftime('%d%H%M'))

    def test_create(self):
        employee = Employee.all(max_results=1, qb=self.qb_client)[0]

        time_activity = TimeActivity()
        time_activity.NameOf = "Employee"

        time_activity.EmployeeRef = employee.to_ref()
        time_activity.Description = "Test description"
        time_activity.StartTime = qb_datetime_utc_offset_format(datetime(2016, 7, 22, 10, 0), '-07:00')
        time_activity.EndTime = qb_datetime_utc_offset_format(datetime(2016, 7, 22, 11, 0), '-07:00')
        time_activity.save(qb=self.qb_client)

        query_time_activity = TimeActivity.get(time_activity.Id, qb=self.qb_client)

        self.assertEquals(query_time_activity.Id, time_activity.Id)
        self.assertEquals(query_time_activity.NameOf, "Employee")
        self.assertEquals(query_time_activity.Description, "Test description")
        self.assertEquals(query_time_activity.EmployeeRef.value, employee.Id)

        # Quickbooks has issues with returning the correct StartTime and EndTime
        #self.assertEquals(query_time_activity.StartTime, '2016-07-22T10:00:00-07:00')
        #self.assertEquals(query_time_activity.EndTime, '2016-07-22T11:00:00-07:00')

    def test_update(self):
        time_activity = TimeActivity.all(max_results=1, qb=self.qb_client)[0]
        time_activity.Description = "Updated test description"
        time_activity.save(qb=self.qb_client)

        query_time_activity = TimeActivity.get(time_activity.Id, qb=self.qb_client)

        self.assertEquals(query_time_activity.Description, "Updated test description")



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.estimate import Estimate


class EstimateTests(unittest.TestCase):
    def test_unicode(self):
        estimate = Estimate()
        estimate.TotalAmt = 10

        self.assertEquals(str(estimate), "10")

    def test_valid_object_name(self):
        obj = Estimate()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.transfer import Transfer


class TaxAgencyTests(unittest.TestCase):
    def test_unicode(self):
        transfer = Transfer()
        transfer.Amount = 100

        self.assertEquals(str(transfer), "100")

    def test_valid_object_name(self):
        obj = Transfer()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.creditcardpayment_entity import CreditCardPayment


class TaxAgencyTests(unittest.TestCase):
    def test_unicode(self):
        credit_card_payment = CreditCardPayment()
        credit_card_payment.Amount = 100

        self.assertEquals(str(credit_card_payment), "100")

    def test_valid_object_name(self):
        obj = CreditCardPayment()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)



import unittest

from quickbooks.objects.taxagency import TaxAgency


class TaxAgencyTests(unittest.TestCase):
    def test_unicode(self):
        deposit = TaxAgency()
        deposit.DisplayName = "test"

        self.assertEquals(str(deposit), "test")



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.refundreceipt import RefundReceipt


class RefundReceiptTests(unittest.TestCase):
    def test_unicode(self):
        deposit = RefundReceipt()
        deposit.TotalAmt = 100

        self.assertEquals(str(deposit), "100")

    def test_valid_object_name(self):
        obj = RefundReceipt()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)


import unittest

from quickbooks import QuickBooks
from quickbooks.objects.preferences import Preferences


class PreferencesTests(unittest.TestCase):
    def test_unicode(self):
        preferences = Preferences()
        preferences.Id = 137

        self.assertEquals(str(preferences), "Preferences 137")

    def test_valid_object_name(self):
        preferences = Preferences()
        client = QuickBooks()
        result = client.isvalid_object_name(preferences.qbo_object_name)

        self.assertTrue(result)



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.creditmemo import CreditMemo
from quickbooks.objects.detailline import SalesItemLineDetail, \
    DiscountLineDetail, SubtotalLineDetail, DiscountOverride, DescriptionLineDetail


class SalesItemLineDetailTests(unittest.TestCase):
    def test_unicode(self):
        detail = SalesItemLineDetail()
        detail.UnitPrice = 10

        self.assertEquals(str(detail), "10")


class CreditMemoTests(unittest.TestCase):
    def test_unicode(self):
        credit_memo = CreditMemo()
        credit_memo.TotalAmt = 1000

        self.assertEquals(str(credit_memo), "1000")

    def test_valid_object_name(self):
        obj = CreditMemo()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)


class DiscountLineDetailTests(unittest.TestCase):
    def test_init(self):
        discount_detail = DiscountLineDetail()

        self.assertEquals(discount_detail.ClassRef, None)
        self.assertEquals(discount_detail.TaxCodeRef, None)
        self.assertEquals(discount_detail.Discount, None)


class SubtotalLineDetailTests(unittest.TestCase):
    def test_init(self):
        detail = SubtotalLineDetail()

        self.assertEquals(detail.ItemRef, None)


class DiscountOverrideTests(unittest.TestCase):
    def test_init(self):
        discount_detail = DiscountOverride()

        self.assertEquals(discount_detail.PercentBased, False)
        self.assertEquals(discount_detail.DiscountPercent, 0)
        self.assertEquals(discount_detail.DiscountAccountRef, None)
        self.assertEquals(discount_detail.DiscountRef, None)


class DescriptionLineDetailTests(unittest.TestCase):
    def test_init(self):
        detail = DescriptionLineDetail()

        self.assertEquals(detail.ServiceDate, "")
        self.assertEquals(detail.TaxCodeRef, None)



from datetime import datetime

from quickbooks.objects.department import Department
from tests.integration.test_base import QuickbooksTestCase


class DepartmentTest(QuickbooksTestCase):
    def setUp(self):
        super(DepartmentTest, self).setUp()

        self.name = "Test Department {0}".format(datetime.now().strftime('%d%H%M%S'))

    def test_create(self):
        department = Department()
        department.Name = self.name
        department.save(qb=self.qb_client)

        query_department = Department.get(department.Id, qb=self.qb_client)

        self.assertEqual(department.Id, query_department.Id)
        self.assertEqual(query_department.Name, self.name)

    def test_update(self):
        department = Department.all(max_results=1, qb=self.qb_client)[0]
        unique_name = "Test Department {0}".format(datetime.now().strftime('%d%H%M%S'))

        department.Name = unique_name
        department.save(qb=self.qb_client)

        query_department = Department.get(department.Id, qb=self.qb_client)

        self.assertEqual(query_department.Name, unique_name)



from six import python_2_unicode_compatible
from .base import QuickbooksBaseObject
from ..mixins import UpdateMixin
from ..client import QuickBooks


@python_2_unicode_compatible
class TaxRateDetails(QuickbooksBaseObject):
    qbo_object_name = "TaxRateDetails"

    def __init__(self):
        super(TaxRateDetails, self).__init__()
        self.TaxRateName = None
        self.TaxRateId = None
        self.RateValue = None
        self.TaxAgencyId = None
        self.TaxApplicableOn = "Sales"

    def __str__(self):
        return self.TaxRateName


@python_2_unicode_compatible
class TaxService(QuickbooksBaseObject, UpdateMixin):
    """
    QBO definition: The TaxService endpoint allows you to perform the following actions:

     - Create a new tax code and specify a list of existing tax rates to be associated to that tax code. To retrieve a
       list of existing tax codes, query the TaxCode endpoint.
     - Create a new tax rate dynamically. To retrieve a list of existing tax rates, query the TaxRate endpoint.

    Tax agency entities on tax rates created via the TaxService endpoint are referenced by id only. That is, you
    cannot create new tax agencies via the TaxService endpoint.
    """

    list_dict = {
        "TaxRateDetails": TaxRateDetails
    }

    qbo_object_name = "TaxService/Taxcode"

    def __init__(self):
        super(TaxService, self).__init__()
        self.TaxCode = None  # Required
        self.TaxCodeId = None  # Readonly - this is the unique database Id (called Id on every other model...)

        self.Id = 0
        self.TaxRateDetails = []

    def __str__(self):
        return self.TaxCode

    def save(self, qb=None):
        if not qb:
            qb = QuickBooks()

        if self.TaxCodeId and self.TaxCodeId > 0:
            json_data = qb.update_object(self.qbo_object_name, self.to_json())
        else:
            json_data = qb.create_object(self.qbo_object_name, self.to_json())

        obj = type(self).from_json(json_data)
        self.TaxCodeId = obj.Id
        self.Id = self.TaxCodeId

        return obj



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.exchangerate import ExchangeRate


class ExchangeRateTests(unittest.TestCase):
    def test_unicode(self):
        exchange_rate = ExchangeRate()
        exchange_rate.SourceCurrencyCode = "EUR"

        self.assertEquals(str(exchange_rate), "EUR")

    def test_valid_object_name(self):
        obj = ExchangeRate()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)



from quickbooks.objects.account import Account
from quickbooks.objects.deposit import Deposit, DepositLine, DepositLineDetail
from tests.integration.test_base import QuickbooksTestCase


class DepositTest(QuickbooksTestCase):
    def test_create(self):
        deposit = Deposit()
        account = Account.filter(AccountType="Bank", max_results=2, qb=self.qb_client)

        account_ref = account[0].to_ref()
        deposit_to_account_ref = account[1].to_ref()

        deposit_line_detail = DepositLineDetail()
        deposit_line_detail.AccountRef = account_ref

        line = DepositLine()
        line.Amount = 20.00
        line.DetailType = "DepositLineDetail"
        line.DepositLineDetail = deposit_line_detail

        deposit.DepositToAccountRef = deposit_to_account_ref
        deposit.Line.append(line)

        deposit.save(qb=self.qb_client)

        query_deposit = Deposit.get(deposit.Id, qb=self.qb_client)

        self.assertEqual(deposit.Id, query_deposit.Id)



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.journalentry import JournalEntry, JournalEntryLine, JournalEntryLineDetail, Entity


class JournalentryTests(unittest.TestCase):
    def test_unicode(self):
        journalentry = JournalEntry()
        journalentry.TotalAmt = 1000

        self.assertEquals(str(journalentry), '1000')

    def test_valid_object_name(self):
        obj = JournalEntry()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)


class JournalEntryLineTests(unittest.TestCase):
    def test_init(self):
        journalentry = JournalEntryLine()

        self.assertEquals(journalentry.DetailType, "JournalEntryLineDetail")
        self.assertEquals(journalentry.JournalEntryLineDetail, None)


class JournalEntryLineDetailTests(unittest.TestCase):
    def test_init(self):
        journalentry = JournalEntryLineDetail()

        self.assertEquals(journalentry.PostingType, "")
        self.assertEquals(journalentry.TaxApplicableOn, "Sales")
        self.assertEquals(journalentry.TaxAmount, 0)
        self.assertEquals(journalentry.BillableStatus, None)
        self.assertEquals(journalentry.Entity, None)
        self.assertEquals(journalentry.AccountRef, None)
        self.assertEquals(journalentry.ClassRef, None)
        self.assertEquals(journalentry.DepartmentRef, None)
        self.assertEquals(journalentry.TaxCodeRef, None)


class EntityTests(unittest.TestCase):
    def test_init(self):
        entity = Entity()

        self.assertEquals(entity.Type, "")
        self.assertEquals(entity.EntityRef, None)



import unittest

from quickbooks.objects.detailline import SalesItemLineDetail, DiscountOverride, DetailLine, SubtotalLineDetail, \
    DiscountLineDetail, SubtotalLine, DescriptionLineDetail, SalesItemLine, DiscountLine, GroupLine, \
    AccountBasedExpenseLineDetail, ItemBasedExpenseLineDetail, DescriptionOnlyLine, ItemBasedExpenseLine


class DetailLineTests(unittest.TestCase):
    def test_unicode(self):
        detail = DetailLine()
        detail.LineNum = 1
        detail.Description = "Product Description"
        detail.Amount = 100

        self.assertEquals(str(detail), "[1] Product Description 100")


class SalesItemLineDetailTests(unittest.TestCase):
    def test_unicode(self):
        sales_detail = SalesItemLineDetail()
        sales_detail.UnitPrice = 10

        self.assertEquals(str(sales_detail), "10")


class DiscountOverrideTests(unittest.TestCase):
    def test_init(self):
        discount_override = DiscountOverride()

        self.assertEquals(discount_override.DiscountPercent, 0)
        self.assertEquals(discount_override.DiscountRef, None)
        self.assertEquals(discount_override.DiscountAccountRef, None)
        self.assertFalse(discount_override.PercentBased)


class DiscountLineDetailTesets(unittest.TestCase):
    def test_init(self):
        discount_detail = DiscountLineDetail()

        self.assertEquals(discount_detail.Discount, None)
        self.assertEquals(discount_detail.ClassRef, None)
        self.assertEquals(discount_detail.TaxCodeRef, None)

class SubtotalLineDetailTest(unittest.TestCase):
    def test_init(self):
        detail = SubtotalLineDetail()

        self.assertEquals(detail.ItemRef, None)


class SubtotalLineTest(unittest.TestCase):
    def test_init(self):
        subtotal_line = SubtotalLine()

        self.assertEquals(subtotal_line.DetailType, "SubTotalLineDetail")
        self.assertEquals(subtotal_line.SubtotalLineDetail, None)


class DescriptionLineDetailTest(unittest.TestCase):
    def test_init(self):
        description_detail = DescriptionLineDetail()

        self.assertEquals(description_detail.ServiceDate, "")
        self.assertEquals(description_detail.TaxCodeRef, None)


class SalesItemLineTest(unittest.TestCase):
    def test_init(self):
        line = SalesItemLine()

        self.assertEquals(line.DetailType, "SalesItemLineDetail")
        self.assertEquals(line.SalesItemLineDetail, None)


class DiscountLineTest(unittest.TestCase):
    def test_init(self):
        line = DiscountLine()

        self.assertEquals(line.DetailType, "DiscountLineDetail")
        self.assertEquals(line.DiscountLineDetail, None)


class GroupLineTest(unittest.TestCase):
    def test_init(self):
        line = GroupLine()

        self.assertEquals(line.DetailType, "GroupLineDetail")
        self.assertEquals(line.GroupLineDetail, None)


class ItemBasedExpenseLineDetailTest(unittest.TestCase):
    def test_init(self):
        detail = ItemBasedExpenseLineDetail()

        self.assertEquals(detail.BillableStatus, None)
        self.assertEquals(detail.UnitPrice, 0)
        self.assertEquals(detail.TaxInclusiveAmt, 0)
        self.assertEquals(detail.Qty, 0)
        self.assertEquals(detail.ItemRef, None)
        self.assertEquals(detail.ClassRef, None)
        self.assertEquals(detail.PriceLevelRef, None)
        self.assertEquals(detail.TaxCodeRef, None)
        self.assertEquals(detail.MarkupInfo, None)
        self.assertEquals(detail.CustomerRef, None)


class ItemBasedExpenseLineTests(unittest.TestCase):
    def test_unicode(self):
        line = ItemBasedExpenseLine()

        self.assertEquals(line.DetailType, "ItemBasedExpenseLineDetail")
        self.assertEquals(line.ItemBasedExpenseLineDetail, None)


class AccountBasedExpenseLineDetailTests(unittest.TestCase):
    def test_unicode(self):
        acct_detail = AccountBasedExpenseLineDetail()
        acct_detail.BillableStatus = "test"

        self.assertEquals(str(acct_detail), "test")


class DescriptionOnlyLineTests(unittest.TestCase):
    def test_unicode(self):
        line = DescriptionOnlyLine()

        self.assertEquals(line.DetailType, "DescriptionOnly")
        self.assertEquals(line.DescriptionLineDetail, None)



from six import python_2_unicode_compatible
from .base import CustomField, Ref, CustomerMemo, Address, EmailAddress, QuickbooksManagedObject, \
    LinkedTxnMixin, QuickbooksTransactionEntity, LinkedTxn
from .tax import TxnTaxDetail
from .detailline import DetailLine, SalesItemLine, GroupLine, DescriptionOnlyLine, DiscountLine, SubtotalLine
from ..mixins import QuickbooksPdfDownloadable, DeleteMixin, SendMixin


@python_2_unicode_compatible
class Estimate(DeleteMixin,
               QuickbooksPdfDownloadable,
               QuickbooksManagedObject,
               QuickbooksTransactionEntity,
               LinkedTxnMixin,
               SendMixin):
    """
    QBO definition: The Estimate represents a proposal for a financial transaction from a business to a customer
    for goods or services proposed to be sold, including proposed pricing.
    """

    class_dict = {
        "BillAddr": Address,
        "ShipAddr": Address,
        "CustomerRef": Ref,
        "TxnTaxDetail": TxnTaxDetail,
        "CustomerMemo": CustomerMemo,
        "BillEmail": EmailAddress,
        "DepartmentRef": Ref,
        "CurrencyRef": Ref,
        "ClassRef": Ref,
        "SalesTermRef": Ref,
        "ShipMethodRef": Ref,
    }

    list_dict = {
        "CustomField": CustomField,
        "LinkedTxn": LinkedTxn,
        "Line": DetailLine,
    }

    detail_dict = {
        "SalesItemLineDetail": SalesItemLine,
        "GroupLineDetail": GroupLine,
        "DescriptionOnly": DescriptionOnlyLine,
        "DiscountLineDetail": DiscountLine,
        "SubTotalLineDetail": SubtotalLine,
    }

    qbo_object_name = "Estimate"

    def __init__(self):
        super(Estimate, self).__init__()
        self.DocNumber = None
        self.TxnDate = None
        self.TxnStatus = None
        self.PrivateNote = None
        self.TotalAmt = 0
        self.ExchangeRate = 1
        self.ApplyTaxAfterDiscount = False
        self.PrintStatus = "NotSet"
        self.EmailStatus = "NotSet"
        self.DueDate = None
        self.ShipDate = None
        self.ExpirationDate = None
        self.AcceptedBy = None
        self.AcceptedDate = None
        self.GlobalTaxCalculation = "TaxExcluded"
        self.BillAddr = None
        self.ShipAddr = None
        self.BillEmail = None
        self.CustomerRef = None
        self.TxnTaxDetail = None
        self.CustomerMemo = None
        self.ClassRef = None
        self.SalesTermRef = None
        self.ShipMethodRef = None

        self.CustomField = []
        self.LinkedTxn = []
        self.Line = []

    def __str__(self):
        return str(self.TotalAmt)



from tests.integration.test_base import QuickbooksTestCase
from datetime import datetime

from quickbooks.objects.detailline import SalesItemLineDetail, \
    DiscountLineDetail, SalesItemLine, DiscountLine
from quickbooks.objects.tax import TxnTaxDetail
from quickbooks.objects.customer import Customer
from quickbooks.objects.base import Address, EmailAddress, CustomerMemo, Ref
from quickbooks.objects.estimate import Estimate

from datetime import datetime

from quickbooks.objects.base import Address, EmailAddress, CustomerMemo, Ref
from quickbooks.objects.customer import Customer
from quickbooks.objects.detailline import SalesItemLineDetail, \
    DiscountLineDetail, SalesItemLine, DiscountLine
from quickbooks.objects.estimate import Estimate
from quickbooks.objects.tax import TxnTaxDetail
from tests.integration.test_base import QuickbooksTestCase


class EstimateTest(QuickbooksTestCase):
    def test_create(self):
        self.customer = Customer.all(max_results=1, qb=self.qb_client)[0]

        estimate = Estimate()
        estimate.TotalAmt = 31.5
        estimate.ApplyTaxAfterDiscount = False
        estimate.PrintStatus = "NeedToPrint"
        estimate.EmailStatus = "NotSet"

        estimate.BillAddr = Address()
        estimate.BillAddr.Line1 = "65 Ocean Dr."
        estimate.BillAddr.City = "Half Moon Bay"
        estimate.BillAddr.CountrySubDivisionCode = "CA"
        estimate.BillAddr.PostalCode = "94213"
        estimate.BillAddr.Lat = "37.4300318"
        estimate.BillAddr.Long = "-122.4336537"

        estimate.ShipAddr = Address()
        estimate.ShipAddr.Id = "2" + datetime.now().strftime('%d%H%M')
        estimate.ShipAddr.Line1 = "65 Ocean Dr."
        estimate.ShipAddr.City = "Half Moon Bay"
        estimate.ShipAddr.CountrySubDivisionCode = "CA"
        estimate.ShipAddr.PostalCode = "94213"
        estimate.ShipAddr.Lat = "37.4300318"
        estimate.ShipAddr.Long = "-122.4336537"

        estimate.BillEmail = EmailAddress()
        estimate.BillEmail.Address = "Cool_Cars@intuit.com"

        estimate.CustomerMemo = CustomerMemo()
        estimate.CustomerMemo.value = "Thank you for your business and have a great day!"

        estimate.CustomerRef = Ref()
        estimate.CustomerRef.value = self.customer.Id
        estimate.CustomerRef.name = self.customer.DisplayName

        estimate.TxnTaxDetail = TxnTaxDetail()
        estimate.TxnTaxDetail.TotalTax = 0

        line = SalesItemLine()
        line.LineNum = 1
        line.Description = "Pest Control Services"
        line.Amount = 35.0

        line.SalesItemLineDetail = SalesItemLineDetail()
        line.SalesItemLineDetail.UnitPrice = 35
        line.SalesItemLineDetail.Qty = 1

        item_ref = Ref()
        item_ref.value = "10"
        item_ref.name = "Pest Control"
        line.SalesItemLineDetail.ItemRef = item_ref

        tax_code_ref = Ref()
        tax_code_ref.value = "NON"
        line.SalesItemLineDetail.TaxCodeRef = tax_code_ref

        estimate.Line.append(line)

        line2 = DiscountLine()
        line2.Amount = 3.5

        line2.DiscountLineDetail = DiscountLineDetail()
        line2.DiscountLineDetail.PercentBased = True
        line2.DiscountLineDetail.DiscountPercent = 10

        line2.DiscountLineDetail.DiscountAccountRef = Ref()
        line2.DiscountLineDetail.DiscountAccountRef.value = "86"
        line2.DiscountLineDetail.DiscountAccountRef.name = "Discounts given"

        line2.DetailType = "DiscountLineDetail"

        estimate.Line.append(line2)

        estimate.save(qb=self.qb_client)

        query_estimate = Estimate.get(estimate.Id, qb=self.qb_client)

        self.assertEqual(query_estimate.Id, estimate.Id)
        self.assertEqual(query_estimate.TotalAmt, estimate.TotalAmt)
        self.assertEqual(query_estimate.ApplyTaxAfterDiscount, estimate.ApplyTaxAfterDiscount)
        self.assertEqual(query_estimate.PrintStatus, estimate.PrintStatus)
        self.assertEqual(query_estimate.EmailStatus, estimate.EmailStatus)
        self.assertEqual(query_estimate.BillAddr.Line1, estimate.BillAddr.Line1)
        self.assertEqual(query_estimate.BillAddr.City, estimate.BillAddr.City)
        self.assertEqual(query_estimate.BillAddr.CountrySubDivisionCode,
                         estimate.BillAddr.CountrySubDivisionCode)
        self.assertEqual(query_estimate.BillAddr.PostalCode, estimate.BillAddr.PostalCode)
        self.assertEqual(query_estimate.ShipAddr.Line1, estimate.ShipAddr.Line1)
        self.assertEqual(query_estimate.ShipAddr.City, estimate.ShipAddr.City)
        self.assertEqual(query_estimate.ShipAddr.CountrySubDivisionCode,
                         estimate.ShipAddr.CountrySubDivisionCode)
        self.assertEqual(query_estimate.ShipAddr.PostalCode, estimate.ShipAddr.PostalCode)
        self.assertEqual(query_estimate.BillEmail.Address, estimate.BillEmail.Address)
        self.assertEqual(query_estimate.CustomerMemo.value, estimate.CustomerMemo.value)
        self.assertEqual(query_estimate.CustomerRef.value, estimate.CustomerRef.value)
        self.assertEqual(query_estimate.CustomerRef.name, estimate.CustomerRef.name)
        self.assertEqual(query_estimate.TxnTaxDetail.TotalTax, estimate.TxnTaxDetail.TotalTax)
        self.assertEqual(query_estimate.Line[0].LineNum, estimate.Line[0].LineNum)
        self.assertEqual(query_estimate.Line[0].Description, estimate.Line[0].Description)
        self.assertEqual(query_estimate.Line[0].Amount, estimate.Line[0].Amount)
        self.assertEqual(query_estimate.Line[0].SalesItemLineDetail.UnitPrice,
                         estimate.Line[0].SalesItemLineDetail.UnitPrice)
        self.assertEqual(query_estimate.Line[0].SalesItemLineDetail.Qty,
                         estimate.Line[0].SalesItemLineDetail.Qty)
        self.assertEqual(query_estimate.Line[2].Amount, estimate.Line[1].Amount)
        self.assertEqual(query_estimate.Line[2].DiscountLineDetail.PercentBased,
                         estimate.Line[1].DiscountLineDetail.PercentBased)
        self.assertEqual(query_estimate.Line[2].DiscountLineDetail.DiscountPercent,
                         estimate.Line[1].DiscountLineDetail.DiscountPercent)
        self.assertEqual(query_estimate.Line[2].DiscountLineDetail.DiscountAccountRef.value,
                         estimate.Line[1].DiscountLineDetail.DiscountAccountRef.value)
        self.assertEqual(query_estimate.Line[2].DiscountLineDetail.DiscountAccountRef.name,
                         estimate.Line[1].DiscountLineDetail.DiscountAccountRef.name)



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.account import Account


class AccountTests(unittest.TestCase):
    def test_unicode(self):
        account = Account()
        account.FullyQualifiedName = "test"

        self.assertEquals(str(account), "test")

    def test_to_ref(self):
        account = Account()
        account.FullyQualifiedName = "test"
        account.Id = 12

        ref = account.to_ref()

        self.assertEquals(ref.name, "test")
        self.assertEquals(ref.type, "Account")
        self.assertEquals(ref.value, 12)

    def test_valid_object_name(self):
        account = Account()
        client = QuickBooks()
        result = client.isvalid_object_name(account.qbo_object_name)

        self.assertTrue(result)


import unittest

from quickbooks import QuickBooks
from quickbooks.objects.deposit import Deposit, DepositLine, CashBackInfo, DepositLineDetail


class DepositTests(unittest.TestCase):
    def test_unicode(self):
        deposit = Deposit()
        deposit.TotalAmt = 100

        self.assertEquals(str(deposit), "100")

    def test_valid_object_name(self):
        obj = Deposit()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)


class DepositLineTests(unittest.TestCase):
    def test_unicode(self):
        deposit = DepositLine()
        deposit.Amount = 100

        self.assertEquals(str(deposit), "100")


class CashBackInfoTests(unittest.TestCase):
    def test_init(self):
        cash_back_info = CashBackInfo()

        self.assertEquals(cash_back_info.Amount, 0)
        self.assertEquals(cash_back_info.Memo, "")
        self.assertEquals(cash_back_info.AccountRef, None)


class DepositLineDetailTests(unittest.TestCase):
    def test_init(self):
        detail = DepositLineDetail()

        self.assertEquals(detail.Entity, None)
        self.assertEquals(detail.ClassRef, None)
        self.assertEquals(detail.AccountRef, None)
        self.assertEquals(detail.PaymentMethodRef, None)
        self.assertEquals(detail.CheckNum, "")
        self.assertEquals(detail.TxnType, None)



from six import python_2_unicode_compatible

from quickbooks.objects.detailline import SalesItemLine, SubtotalLine, DiscountLine, DescriptionOnlyLine, DetailLine
from .base import Address, EmailAddress, Ref, CustomField, CustomerMemo, QuickbooksManagedObject, \
    LinkedTxnMixin, QuickbooksTransactionEntity
from .tax import TxnTaxDetail
from ..mixins import DeleteMixin


@python_2_unicode_compatible
class CreditMemo(DeleteMixin, QuickbooksTransactionEntity, QuickbooksManagedObject, LinkedTxnMixin):
    """
    QBO definition: The CreditMemo is a financial transaction representing a refund or credit of payment or part
    of a payment for goods or services that have been sold.
    """

    class_dict = {
        "BillAddr": Address,
        "ShipAddr": Address,
        "DepartmentRef": Ref,
        "ClassRef": Ref,
        "CustomerRef": Ref,
        "CurrencyRef": Ref,
        "SalesTermRef": Ref,
        "CustomerMemo": CustomerMemo,
        "BillEmail": EmailAddress,
        "TxnTaxDetail": TxnTaxDetail,
        "PaymentMethodRef": Ref,
        "DepositToAccountRef": Ref,
    }

    list_dict = {
        "CustomField": CustomField,
        "Line": DetailLine
    }

    detail_dict = {
        "SalesItemLineDetail": SalesItemLine,
        "SubTotalLineDetail": SubtotalLine,
        "DiscountLineDetail": DiscountLine,
        "DescriptionLineDetail": DescriptionOnlyLine
    }

    qbo_object_name = "CreditMemo"

    def __init__(self):
        super(CreditMemo, self).__init__()
        self.RemainingCredit = 0
        self.ExchangeRate = 0
        self.DocNumber = ""
        self.TxnDate = ""
        self.PrivateNote = ""
        self.TotalAmt = 0
        self.ApplyTaxAfterDiscount = ""
        self.PrintStatus = "NotSet"
        self.EmailStatus = "NotSet"
        self.Balance = 0
        self.GlobalTaxCalculation = "TaxExcluded"

        self.BillAddr = None
        self.ShipAddr = None
        self.ClassRef = None
        self.DepartmentRef = None
        self.CustomerRef = None
        self.CurrencyRef = None
        self.CustomerMemo = None
        self.BillEmail = None
        self.TxnTaxDetail = None
        self.SalesTermRef = None

        self.CustomField = []
        self.Line = []

    def __str__(self):
        return str(self.TotalAmt)



from six import python_2_unicode_compatible

from quickbooks.mixins import PrefMixin, UpdateNoIdMixin
from .base import QuickbooksBaseObject, QuickbooksTransactionEntity, Ref, EmailAddress


@python_2_unicode_compatible
class PreferencesCustomField(QuickbooksBaseObject):
    def __init__(self):
        self.Type = ""
        self.Name = ""
        self.StringValue = ""
        self.BooleanValue = ""

    def __str__(self):
        return self.Name


class PreferencesCustomFieldGroup(QuickbooksBaseObject):
    list_dict = {
        "CustomField": PreferencesCustomField
    }

    def __init__(self):
        super().__init__()


class EmailMessageType(QuickbooksBaseObject):
    def __init__(self):
        super().__init__()
        self.Message = ""
        self.Subject = ""


class EmailMessagesPrefs(QuickbooksBaseObject):
    class_dict = {
        "InvoiceMessage": EmailMessageType,
        "EstimateMessage": EmailMessageType,
        "SalesReceiptMessage": EmailMessageType,
        "StatementMessage": EmailMessageType,
    }

    def __init__(self):
        super().__init__()
        self.InvoiceMessage = None
        self.EstimateMessage = None
        self.SalesReceiptMessage = None
        self.StatementMessage = None


class ProductAndServicesPrefs(QuickbooksBaseObject):

    def __init__(self):
        super().__init__()
        self.QuantityWithPriceAndRate = True
        self.ForPurchase = True
        self.QuantityOnHand = True
        self.ForSales = True
        self.RevenueRecognition = True
        self.RevenueRecognitionFrequency = ""


class ReportPrefs(QuickbooksBaseObject):

    def __init__(self):
        super().__init__()
        self.ReportBasis = "Accrual"  # or "Cash"
        self.CalcAgingReportFromTxnDate = False  # read only


class AccountingInfoPrefs(QuickbooksBaseObject):
    def __init__(self):
        super().__init__()
        self.FirstMonthOfFiscalYear = "January"  # read only
        self.UseAccountNumbers = True  # read only
        self.TaxYearMonth = "January"  # read only
        self.ClassTrackingPerTxn = False
        self.TrackDepartments = False
        self.TaxForm = "6"
        # Possible values include: Clients, Customers, Donors, Guests, Members, Patients, Tenants.
        self.CustomerTerminology = ""  # Customers
        self.BookCloseDate = ""  # e.g. "2018-12-31"
        # Possible values include: Business, Department, Division, Location, Property, Store, Territory
        self.DepartmentTerminology = ""  # Location
        self.ClassTrackingPerTxnLine = True


class ClassTrackingPerTxnLine(QuickbooksBaseObject):
    def __init__(self):
        super().__init__()
        self.ReportBasis = "Accrual"  # or "Cash"
        self.CalcAgingReportFromTxnDate = False  # read only


class SalesFormsPrefs(QuickbooksBaseObject):
    class_dict = {
        "DefaultTerms": Ref,
        "SalesEmailBcc": EmailAddress,
        "SalesEmailCc": EmailAddress
    }
    detail_dict = {
        "CustomField": PreferencesCustomFieldGroup
    }

    def __init__(self):
        super().__init__()
        self.ETransactionPaymentEnabled = False
        self.CustomTxnNumbers = False
        self.AllowShipping = False
        self.AllowServiceDate = False
        self.ETransactionEnabledStatus = ""  # e.g. "NotApplicable"
        self.DefaultCustomerMessage = ""  # e.g. "Thank you for your business and have a great day!"
        self.EmailCopyToCompany = False
        self.AllowEstimates = True
        self.DefaultTerms = None
        self.AllowDiscount = True
        self.DefaultDiscountAccount = ""
        self.DefaultShippingAccount = False
        self.AllowDeposit = True
        self.AutoApplyPayments = True
        self.IPNSupportEnabled = False
        self.AutoApplyCredit = True
        self.CustomField = None
        self.UsingPriceLevels = False
        self.ETransactionAttachPDF = False
        self.UsingProgressInvoicing = False
        self.EstimateMessage = ""

        self.DefaultTerms = None
        self.CustomField = None
        self.SalesEmailBcc = None
        self.SalesEmailCc = None


class VendorAndPurchasesPrefs(QuickbooksBaseObject):
    class_dict = {
        "DefaultTerms": Ref,
        "DefaultMarkupAccount": Ref
    }
    detail_dict = {
        "POCustomField": PreferencesCustomFieldGroup
    }

    def __init__(self):
        super().__init__()
        self.BillableExpenseTracking = True
        self.TrackingByCustomer = True
        self.TPAREnabled = True

        self.POCustomField = None
        self.DefaultMarkupAccount = None
        self.DefaultTerms = None


class TaxPrefs(QuickbooksBaseObject):
    class_dict = {
        "TaxGroupCodeRef": Ref
    }

    def __init__(self):
        super().__init__()
        self.TaxGroupCodeRef = None
        self.UsingSalesTax = True
        self.PartnerTaxEnabled = True


class NameValue(QuickbooksBaseObject):
    def __init__(self):
        super().__init__()
        self.Name = ""
        self.Value = ""


class OtherPrefs(QuickbooksBaseObject):
    list_dict = {
        "NameValue": NameValue
    }

    def __init__(self):
        super().__init__()
        self.NameValue = None


class TimeTrackingPrefs(QuickbooksBaseObject):
    def __init__(self):
        super().__init__()
        self.WorkWeekStartDate = ""  # e.g. "Monday"
        self.MarkTimeEntriesBillable = True
        self.ShowBillRateToAll = False
        self.UseServices = True
        self.BillCustomers = True


class CurrencyPrefs(QuickbooksBaseObject):
    class_dict = {
        "HomeCurrency": Ref
    }

    def __init__(self):
        super().__init__()
        self.HomeCurrency = None
        self.MultiCurrencyEnabled = False


@python_2_unicode_compatible
class Preferences(PrefMixin, UpdateNoIdMixin, QuickbooksTransactionEntity):
    """
    QBO definition: The Preferences resource represents a set of company preferences that
    control application behavior in QuickBooks Online.
    They are mostly exposed as read-only through the Preferences endpoint with only a very small subset of them
    available as writable. Preferences are not necessarily honored when making requests via the QuickBooks API
    because a lot of them control UI behavior in the application and may not be applicable for apps.
    """

    class_dict = {
        'EmailMessagesPrefs': EmailMessagesPrefs,
        'ProductAndServicesPrefs': ProductAndServicesPrefs,
        'ReportPrefs': ReportPrefs,
        'AccountingInfoPrefs': AccountingInfoPrefs,
        'SalesFormsPrefs': SalesFormsPrefs,
        'VendorAndPurchasesPrefs': VendorAndPurchasesPrefs,
        'TaxPrefs': TaxPrefs,
        'OtherPrefs': OtherPrefs,
        'TimeTrackingPrefs': TimeTrackingPrefs,
        'CurrencyPrefs': CurrencyPrefs,
    }

    qbo_object_name = "Preferences"

    def __init__(self):
        super().__init__()
        self.EmailMessagesPrefs = None
        self.ProductAndServicesPrefs = None
        self.ReportPrefs = None
        self.AccountingInfoPrefs = None
        self.SalesFormsPrefs = None
        self.VendorAndPurchasesPrefs = None
        self.TaxPrefs = None
        self.OtherPrefs = None
        self.TimeTrackingPrefs = None
        self.CurrencyPrefs = None

    def __str__(self):
        return 'Preferences {0}'.format(self.Id)



from datetime import datetime

from quickbooks.objects.base import Address, PhoneNumber, EmailAddress, WebAddress
from quickbooks.objects.vendor import Vendor
from tests.integration.test_base import QuickbooksTestCase


class VendorTest(QuickbooksTestCase):
    def setUp(self):
        super(VendorTest, self).setUp()

        self.account_number = datetime.now().strftime('%d%H%M')
        self.name = "Test Vendor {0}".format(self.account_number)

    def test_create(self):
        vendor = Vendor()

        vendor.TaxIdentifier = '99-9999999'
        vendor.AcctNum = self.account_number
        vendor.Title = 'Ms.'
        vendor.GivenName = 'First'
        vendor.FamilyName = 'Last'
        vendor.Suffix = 'Sr.'
        vendor.CompanyName = self.name
        vendor.DisplayName = self.name
        vendor.PrintOnCheckName = self.name

        vendor.BillAddr = Address()
        vendor.BillAddr.Line1 = "123 Main"
        vendor.BillAddr.Line2 = "Apartment 1"
        vendor.BillAddr.City = "City"
        vendor.BillAddr.Country = "U.S.A"
        vendor.BillAddr.CountrySubDivisionCode = "CA"
        vendor.BillAddr.PostalCode = "94030"

        vendor.PrimaryPhone = PhoneNumber()
        vendor.PrimaryPhone.FreeFormNumber = '555-555-5555'

        vendor.PrimaryEmailAddr = EmailAddress()
        vendor.PrimaryEmailAddr.Address = 'test@email.com'

        vendor.WebAddr = WebAddress()
        vendor.WebAddr.URI = 'http://testurl.com'

        vendor.save(qb=self.qb_client)

        query_vendor = Vendor.get(vendor.Id, qb=self.qb_client)

        self.assertEquals(query_vendor.Id, vendor.Id)

        self.assertEquals(query_vendor.AcctNum, self.account_number)
        self.assertEquals(query_vendor.Title, 'Ms.')
        self.assertEquals(query_vendor.GivenName, 'First')
        self.assertEquals(query_vendor.FamilyName, 'Last')
        self.assertEquals(query_vendor.Suffix, 'Sr.')
        self.assertEquals(query_vendor.CompanyName, self.name)
        self.assertEquals(query_vendor.DisplayName, self.name)
        self.assertEquals(query_vendor.PrintOnCheckName, self.name)

        self.assertEquals(query_vendor.BillAddr.Line1, "123 Main")
        self.assertEquals(query_vendor.BillAddr.Line2, "Apartment 1")
        self.assertEquals(query_vendor.BillAddr.City, "City")
        self.assertEquals(query_vendor.BillAddr.Country, "U.S.A")
        self.assertEquals(query_vendor.BillAddr.CountrySubDivisionCode, "CA")
        self.assertEquals(query_vendor.BillAddr.PostalCode, "94030")
        self.assertEquals(query_vendor.PrimaryPhone.FreeFormNumber, '555-555-5555')
        self.assertEquals(query_vendor.PrimaryEmailAddr.Address, 'test@email.com')
        self.assertEquals(query_vendor.WebAddr.URI, 'http://testurl.com')

    def update_vendor(self):
        vendor = Vendor.all(max_results=1, qb=self.qb_client)[0]

        vendor.GivenName = 'Updated Name'
        vendor.FamilyName = 'Updated Lastname'

        vendor.save(qb=self.qb_client)

        query_vendor = Vendor.get(vendor.Id, qb=self.qb_client)
        self.assertEquals(query_vendor.GivenName, 'Updated Name')
        self.assertEquals(query_vendor.FamilyName, 'Updated Lastname')



import unittest

from quickbooks import QuickBooks
from quickbooks.objects.payment import PaymentLine, Payment


class PaymentLineTests(unittest.TestCase):
    def test_unicode(self):
        payment_line = PaymentLine()
        payment_line.Amount = 100

        self.assertEquals(str(payment_line), "100")


class PaymentTests(unittest.TestCase):
    def test_unicode(self):
        payment = Payment()
        payment.TotalAmt = 1000

        self.assertEquals(str(payment), '1000')

    def test_valid_object_name(self):
        obj = Payment()
        client = QuickBooks()
        result = client.isvalid_object_name(obj.qbo_object_name)

        self.assertTrue(result)


from .account import Account
from .attachable import Attachable
from .base import (
    Address, PhoneNumber, EmailAddress, WebAddress, Ref, CustomField,
    LinkedTxn, CustomerMemo, MarkupInfo, AttachableRef
)
from .bill import Bill
from .billpayment import (
    CheckPayment, BillPaymentCreditCard, BillPaymentLine, BillPayment
)
from .budget import BudgetDetail, Budget
from .company_info import CompanyInfo
from .creditcardpayment import (
    CreditChargeInfo, CreditChargeResponse, CreditCardPayment
)
from .creditmemo import CreditMemo
from .customer import Customer
from .department import Department
from .deposit import (
    CashBackInfo, DepositLineDetail, DepositLine, Deposit
)
from .detailline import (
    DetailLine, DiscountOverride, DiscountLineDetail, DiscountLine,
    SubtotalLineDetail, SubtotalLine, DescriptionLineDetail, DescriptionOnlyLine,
    SalesItemLineDetail, SalesItemLine, GroupLineDetail, GroupLine,
    AccountBasedExpenseLineDetail, AccountBasedExpenseLine,
    TDSLineDetail, TDSLine, ItemBasedExpenseLineDetail, ItemBasedExpenseLine,

)
from .employee import Employee
from .estimate import Estimate
from .invoice import DeliveryInfo, Invoice
from .item import Item
from .journalentry import (
    Entity, JournalEntryLineDetail, JournalEntryLine, JournalEntry
)
from .payment import PaymentLine, Payment
from .paymentmethod import PaymentMethod
from .preferences import (
    AccountingInfoPrefs, ClassTrackingPerTxnLine, CurrencyPrefs,
    EmailMessageType, EmailMessagesPrefs, OtherPrefs, Preferences,
    ProductAndServicesPrefs, ReportPrefs, SalesFormsPrefs,
    VendorAndPurchasesPrefs, TaxPrefs, TimeTrackingPrefs,
)
from .purchase import Purchase
from .purchaseorder import PurchaseOrder
from .refundreceipt import RefundReceipt
from .salesreceipt import SalesReceipt
from .tax import TaxLineDetail, TaxLine, TxnTaxDetail
from .taxagency import TaxAgency
from .taxcode import TaxRateDetail, TaxRateList, TaxCode
from .taxrate import TaxRate
from .taxservice import TaxRateDetails, TaxService
from .term import Term
from .timeactivity import TimeActivity
from .trackingclass import Class
from .transfer import Transfer
from .vendor import ContactInfo, Vendor
from .vendorcredit import VendorCredit



from datetime import datetime

from quickbooks.objects import RefundReceipt, DetailLine, CustomerMemo, SalesItemLineDetail, \
    Account, Item
from quickbooks.objects.refundreceipt import RefundReceiptCheckPayment
from tests.integration.test_base import QuickbooksTestCase


class PaymentTest(QuickbooksTestCase):
    def setUp(self):
        super(PaymentTest, self).setUp()

        self.account_number = datetime.now().strftime('%d%H%M')
        self.name = "Test Account {0}".format(self.account_number)

    def test_create(self):
        refund_receipt = RefundReceipt()
        refund_receipt.DocNumber = "DocNum123"
        refund_receipt.TotalAmt = 100
        refund_receipt.Balance = 100
        refund_receipt.PrivateNote = "Private Note"
        refund_receipt.PaymentType = "Check"

        memo = CustomerMemo()
        memo.value = "Customer Memo"
        refund_receipt.CustomerMemo = memo

        refund_receipt.CheckPayment = RefundReceiptCheckPayment()
        refund_receipt.CheckPayment.CheckNum = "1001"
        refund_receipt.CheckPayment.NameOnAcct = "John Smith"
        refund_receipt.CheckPayment.AcctNum = "0000000000"
        refund_receipt.CheckPayment.BankName = "Bank"

        item = Item.all(max_results=1, qb=self.qb_client)[0]
        line = DetailLine()
        line.DetailType = "SalesItemLineDetail"
        line.Amount = 200
        line.SalesItemLineDetail = SalesItemLineDetail()
        line.SalesItemLineDetail.ItemRef = item.to_ref()
        refund_receipt.Line.append(line)

        account = Account.where("Name = 'checking'", max_results=1, qb=self.qb_client)[0]
        refund_receipt.DepositToAccountRef = account.to_ref()

        refund_receipt.save(qb=self.qb_client)

        query_refund_receipt = RefundReceipt.get(refund_receipt.Id, qb=self.qb_client)

        self.assertEqual(query_refund_receipt.DocNumber, refund_receipt.DocNumber)
        self.assertEqual(query_refund_receipt.Line[0].Amount, 200)
        self.assertEqual(refund_receipt.DepositToAccountRef.value, account.Id)



from six import python_2_unicode_compatible
from .base import QuickbooksBaseObject, Ref, CustomField, LinkedTxn, MarkupInfo


@python_2_unicode_compatible
class DetailLine(QuickbooksBaseObject):
    list_dict = {
        "LinkedTxn": LinkedTxn,
        "CustomField": CustomField,
    }

    def __init__(self):
        super(DetailLine, self).__init__()
        self.Id = None
        self.LineNum = 0
        self.Description = None
        self.Amount = 0
        self.DetailType = ""
        self.LinkedTxn = []
        self.CustomField = []

    def __str__(self):
        return "[{0}] {1} {2}".format(self.LineNum, self.Description, self.Amount)


class DiscountOverride(QuickbooksBaseObject):
    class_dict = {
        "DiscountRef": Ref,
        "DiscountAccountRef": Ref,
    }

    qbo_object_name = "DiscountOverride"

    def __init__(self):
        super(DiscountOverride, self).__init__()
        self.PercentBased = False
        self.DiscountPercent = 0
        self.DiscountRef = None
        self.DiscountAccountRef = None


class DiscountLineDetail(QuickbooksBaseObject):
    class_dict = {
        "Discount": DiscountOverride,
        "ClassRef": Ref,
        "TaxCodeRef": Ref,
        "DiscountAccountRef": Ref
    }

    def __init__(self):
        super(DiscountLineDetail, self).__init__()

        self.Discount = None
        self.ClassRef = None
        self.TaxCodeRef = None
        self.PercentBased = False
        self.DiscountPercent = 0


class DiscountLine(DetailLine):
    class_dict = {
        "DiscountLineDetail": DiscountLineDetail
    }

    def __init__(self):
        super(DiscountLine, self).__init__()
        self.DetailType = "DiscountLineDetail"
        self.DiscountLineDetail = None


class SubtotalLineDetail(QuickbooksBaseObject):
    class_dict = {
        "ItemRef": Ref
    }

    def __init__(self):
        super(SubtotalLineDetail, self).__init__()
        self.ItemRef = None


class SubtotalLine(DetailLine):
    class_dict = {
        "SubtotalLineDetail": SubtotalLineDetail
    }

    def __init__(self):
        super(SubtotalLine, self).__init__()
        self.DetailType = "SubTotalLineDetail"
        self.SubtotalLineDetail = None


class DescriptionLineDetail(QuickbooksBaseObject):
    class_dict = {
        "TaxCodeRef": Ref
    }

    def __init__(self):
        super(DescriptionLineDetail, self).__init__()
        self.ServiceDate = ""
        self.TaxCodeRef = None


@python_2_unicode_compatible
class SalesItemLineDetail(QuickbooksBaseObject):
    class_dict = {
        "ItemRef": Ref,
        "ClassRef": Ref,
        "TaxCodeRef": Ref,
        "PriceLevelRef": Ref,
        "MarkupInfo": MarkupInfo,
    }

    def __init__(self):
        super(SalesItemLineDetail, self).__init__()
        self.UnitPrice = 0
        self.Qty = 0
        self.ServiceDate = ""
        self.TaxInclusiveAmt = 0

        self.MarkupInfo = None
        self.ItemRef = None
        self.ClassRef = None
        self.TaxCodeRef = None
        self.PriceLevelRef = None

    def __str__(self):
        return str(self.UnitPrice)


class SalesItemLine(DetailLine):
    class_dict = {
        "SalesItemLineDetail": SalesItemLineDetail
    }

    def __init__(self):
        super(SalesItemLine, self).__init__()
        self.DetailType = "SalesItemLineDetail"
        self.SalesItemLineDetail = None


class GroupLineDetail(QuickbooksBaseObject):
    pass


class GroupLine(DetailLine):
    class_dict = {
        "GroupLineDetail": GroupLineDetail
    }

    def __init__(self):
        super(GroupLine, self).__init__()
        self.DetailType = "GroupLineDetail"
        self.GroupLineDetail = None


class DescriptionOnlyLine(DetailLine):
    class_dict = {
        "DescriptionLineDetail": DescriptionLineDetail
    }

    def __init__(self):
        super(DescriptionOnlyLine, self).__init__()
        self.DetailType = "DescriptionOnly"
        self.DescriptionLineDetail = None


@python_2_unicode_compatible
class AccountBasedExpenseLineDetail(QuickbooksBaseObject):
    class_dict = {
        "CustomerRef": Ref,
        "AccountRef": Ref,
        "TaxCodeRef": Ref,
        "ClassRef": Ref,
        "MarkupInfo": MarkupInfo,
    }

    def __init__(self):
        super(AccountBasedExpenseLineDetail, self).__init__()
        self.BillableStatus = None
        self.TaxAmount = 0
        self.TaxInclusiveAmt = 0

        self.CustomerRef = None
        self.AccountRef = None
        self.TaxCodeRef = None
        self.ClassRef = None

    def __str__(self):
        return self.BillableStatus


class AccountBasedExpenseLine(DetailLine):
    class_dict = {
        "AccountBasedExpenseLineDetail": AccountBasedExpenseLineDetail
    }

    def __init__(self):
        super(AccountBasedExpenseLine, self).__init__()

        self.DetailType = "AccountBasedExpenseLineDetail"
        self.AccountBasedExpenseLineDetail = None


@python_2_unicode_compatible
class TDSLineDetail(QuickbooksBaseObject):
    def __init__(self):
        super(TDSLineDetail, self).__init__()
        self.TDSSectionTypeId = None

    def __str__(self):
        return self.TDSSectionTypeId


class TDSLine(DetailLine):
    class_dict = {
        "TDSLineDetail": TDSLineDetail
    }

    def __init__(self):
        super(TDSLine, self).__init__()

        self.DetailType = "TDSLineDetail"
        self.TDSLineDetail = None


class ItemBasedExpenseLineDetail(QuickbooksBaseObject):
    class_dict = {
        "ItemRef": Ref,
        "ClassRef": Ref,
        "PriceLevelRef": Ref,
        "TaxCodeRef": Ref,
        "CustomerRef": Ref,
        "MarkupInfo": MarkupInfo
    }

    def __init__(self):
        super(ItemBasedExpenseLineDetail, self).__init__()
        self.BillableStatus = None
        self.UnitPrice = 0
        self.TaxInclusiveAmt = 0
        self.Qty = 0
        self.ItemRef = None
        self.ClassRef = None
        self.PriceLevelRef = None
        self.TaxCodeRef = None
        self.MarkupInfo = None
        self.CustomerRef = None


class ItemBasedExpenseLine(DetailLine):
    class_dict = {
        "ItemBasedExpenseLineDetail": ItemBasedExpenseLineDetail
    }

    def __init__(self):
        super(ItemBasedExpenseLine, self).__init__()

        self.DetailType = "ItemBasedExpenseLineDetail"
        self.ItemBasedExpenseLineDetail = None



from datetime import datetime

from quickbooks.objects.base import Address, PhoneNumber
from quickbooks.objects.employee import Employee
from tests.integration.test_base import QuickbooksTestCase


class EmployeeTest(QuickbooksTestCase):
    def test_create(self):
        employee = Employee()
        employee.SSN = "444-55-6666"
        employee.GivenName = "John"
        employee.FamilyName = "Smith {0}".format(datetime.now().strftime('%d%H%M%S'))

        employee.PrimaryAddr = Address()
        employee.PrimaryAddr.Line1 = "45 N. Elm Street"
        employee.PrimaryAddr.City = "Middlefield"
        employee.PrimaryAddr.CountrySubDivisionCode = "CA"
        employee.PrimaryAddr.PostalCode = "93242"

        employee.PrimaryPhone = PhoneNumber()
        employee.PrimaryPhone.FreeFormNumber = "408-525-1234"
        employee.save(qb=self.qb_client)

        query_employee = Employee.get(employee.Id, qb=self.qb_client)

        self.assertEqual(query_employee.Id, employee.Id)
        self.assertEqual(query_employee.SSN, "XXX-XX-XXXX")
        self.assertEqual(query_employee.GivenName, employee.GivenName)
        self.assertEqual(query_employee.FamilyName, employee.FamilyName)
        self.assertEqual(query_employee.PrimaryAddr.Line1, employee.PrimaryAddr.Line1)
        self.assertEqual(query_employee.PrimaryAddr.City, employee.PrimaryAddr.City)
        self.assertEqual(query_employee.PrimaryAddr.CountrySubDivisionCode, employee.PrimaryAddr.CountrySubDivisionCode)
        self.assertEqual(query_employee.PrimaryAddr.PostalCode, employee.PrimaryAddr.PostalCode)
        self.assertEqual(query_employee.PrimaryPhone.FreeFormNumber, employee.PrimaryPhone.FreeFormNumber)

    def test_update(self):
        employee = Employee.all(max_results=1, qb=self.qb_client)[0]
        unique_name = employee.FamilyName = "Smith Updated {0}".format(datetime.now().strftime('%d%H%M%S'))

        employee.FamilyName = unique_name
        employee.save(qb=self.qb_client)

        query_employee = Employee.get(employee.Id, qb=self.qb_client)

        self.assertEqual(query_employee.FamilyName, unique_name)



from quickbooks.objects.account import Account
from quickbooks.objects.customer import Customer
from quickbooks.objects.detailline import ItemBasedExpenseLine, ItemBasedExpenseLineDetail
from quickbooks.objects.item import Item
from quickbooks.objects.purchaseorder import PurchaseOrder
from quickbooks.objects.taxcode import TaxCode
from quickbooks.objects.vendor import Vendor
from tests.integration.test_base import QuickbooksTestCase


class PurchaseOrderTest(QuickbooksTestCase):
    def test_create(self):
        customer = Customer.all(max_results=1, qb=self.qb_client)[0]
        taxcode = TaxCode.all(max_results=1, qb=self.qb_client)[0]
        item = Item.filter(Type='Inventory', max_results=1, qb=self.qb_client)[0]
        vendor = Vendor.all(max_results=1, qb=self.qb_client)[0]
        account = Account.all(max_results=1, qb=self.qb_client)[0]

        purchaseorder = PurchaseOrder()

        detail_line = ItemBasedExpenseLine()
        detail_line.Amount = 100
        detail_line.ItemBasedExpenseLineDetail = ItemBasedExpenseLineDetail()
        detail_line.ItemBasedExpenseLineDetail.BillableStatus = "NotBillable"
        detail_line.ItemBasedExpenseLineDetail.UnitPrice = 100
        detail_line.ItemBasedExpenseLineDetail.Qty = 1
        detail_line.ItemBasedExpenseLineDetail.CustomerRef = customer.to_ref()
        detail_line.ItemBasedExpenseLineDetail.TaxCodeRef = taxcode.to_ref()
        detail_line.ItemBasedExpenseLineDetail.ItemRef = item.to_ref()

        purchaseorder.Line.append(detail_line)
        purchaseorder.VendorRef = vendor.to_ref()
        purchaseorder.APAccountRef = account.to_ref()
        purchaseorder.TotalAmt = 100

        #print purchaseorder.to_json()
        purchaseorder.save(qb=self.qb_client)

        query_purchaseorder = PurchaseOrder.get(purchaseorder.Id, qb=self.qb_client)

        self.assertEquals(query_purchaseorder.VendorRef.value, vendor.Id)
        self.assertEquals(query_purchaseorder.APAccountRef.value, account.Id)
        self.assertEquals(query_purchaseorder.TotalAmt, 100)

        query_detail_line = query_purchaseorder.Line[0]

        self.assertEquals(query_detail_line.Amount, 100)
        self.assertEquals(query_detail_line.ItemBasedExpenseLineDetail.UnitPrice, 100)
        self.assertEquals(query_detail_line.ItemBasedExpenseLineDetail.Qty, 1)
        self.assertEquals(query_detail_line.ItemBasedExpenseLineDetail.CustomerRef.value, customer.Id)
        self.assertEquals(query_detail_line.ItemBasedExpenseLineDetail.TaxCodeRef.value, taxcode.Name)
        self.assertEquals(query_detail_line.ItemBasedExpenseLineDetail.ItemRef.value, item.Id)





from quickbooks.objects.taxcode import TaxCode
from tests.integration.test_base import QuickbooksTestCase


class TaxCodeTest(QuickbooksTestCase):
    def test_get_all(self):
        tax_codes = TaxCode.all(max_results=1, qb=self.qb_client)

        # KNOWN Quickbooks bug - TaxCode query returns 3 extra items:
        # https://intuitdeveloper.lc.intuit.com/questions/1398164-setting-maxresults-on-taxcode-query-returns-incorrect-number-of-records

        self.assertEquals(len(tax_codes), 4)




import unittest

from quickbooks.helpers import qb_date_format, qb_datetime_format, qb_datetime_utc_offset_format
from datetime import datetime, date


class HelpersTests(unittest.TestCase):
    def test_qb_date_format(self):
        result = qb_date_format(date(2016, 7, 22))
        self.assertEquals(result, '2016-07-22')

    def test_qb_datetime_format(self):
        result = qb_datetime_format(datetime(2016, 7, 22, 10, 35, 00))
        self.assertEquals(result, '2016-07-22T10:35:00')

    def test_qb_datetime_utc_offset_format(self):
        result = qb_datetime_utc_offset_format(datetime(2016, 7, 22, 10, 35, 00), '-06:00')
        self.assertEquals(result, '2016-07-22T10:35:00-06:00')



import unittest

from quickbooks.objects.base import CustomerMemo, CustomField, Ref, WebAddress, EmailAddress, PhoneNumber, \
    Address, LinkedTxn, MetaData, MarkupInfo, AttachableRef, LinkedTxnMixin

from quickbooks.objects.deposit import Deposit


class AddressTests(unittest.TestCase):
    def test_unicode(self):
        address = Address()
        address.Line1 = "123 Main"
        address.City = "Joplin"
        address.CountrySubDivisionCode = "MO"
        address.PostalCode = "12345"

        self.assertEquals(str(address), "123 Main Joplin, MO 12345")


class PhoneNumberTests(unittest.TestCase):
    def test_unicode(self):
        number = PhoneNumber()
        number.FreeFormNumber = "555-555-5555"

        self.assertEquals(str(number), "555-555-5555")


class EmailAddressTests(unittest.TestCase):
    def test_unicode(self):
        email = EmailAddress()
        email.Address = "email@gmail.com"

        self.assertEquals(str(email), "email@gmail.com")


class WebAddressTests(unittest.TestCase):
    def test_unicode(self):
        url = WebAddress()
        url.URI = "www.website.com"

        self.assertEquals(str(url), "www.website.com")


class RefTests(unittest.TestCase):
    def test_unicode(self):
        ref = Ref()
        ref.type = "type"
        ref.name = "test"
        ref.value = 1

        self.assertEquals(str(ref), "test")


class CustomFieldTests(unittest.TestCase):
    def test_unicode(self):
        custom = CustomField()
        custom.Name = "name"

        self.assertEquals(str(custom), "name")


class CustomerMemoTests(unittest.TestCase):
    def test_unicode(self):
        memo = CustomerMemo()
        memo.value = "value"

        self.assertEquals(str(memo), "value")


class LinkedTxnTests(unittest.TestCase):
    def test_unicode(self):
        linked = LinkedTxn()
        linked.TxnId = 1

        self.assertEquals(str(linked), "1")


class MetaDataTests(unittest.TestCase):
    def test_unicode(self):
        meta = MetaData()
        meta.CreateTime = "1/1/2000"

        self.assertEquals(str(meta), "Created 1/1/2000")


class MarkupInfoTests(unittest.TestCase):
    def test_init(self):
        markup = MarkupInfo()

        self.assertEquals(markup.PercentBased, False)
        self.assertEquals(markup.Value, 0)
        self.assertEquals(markup.Percent, 0)
        self.assertEquals(markup.PriceLevelRef, None)


class AttachableRefTests(unittest.TestCase):
    def test_init(self):
        attachable = AttachableRef()
        attachable.Name = "test"
        attachable.IncludeOnSend = False
        attachable.Inactive = False
        attachable.NoRefOnly = False

        self.assertEquals(attachable.LineInfo, None)
        self.assertEquals(attachable.IncludeOnSend, False)
        self.assertEquals(attachable.Inactive, False)
        self.assertEquals(attachable.NoRefOnly, False)
        self.assertEquals(attachable.EntityRef, None)


class LinkedTxnMixinTests(unittest.TestCase):
    def test_to_linked_txn(self):

        deposit = Deposit()
        deposit.Id = 100

        linked_txn = deposit.to_linked_txn()

        self.assertEquals(linked_txn.TxnId, 100)
        self.assertEquals(linked_txn.TxnType, "Deposit")
        self.assertEquals(linked_txn.TxnLineId, 1)



from six import python_2_unicode_compatible

from quickbooks.objects import CreditCardPayment
from .base import Ref, CustomField, QuickbooksManagedObject, \
    LinkedTxnMixin, QuickbooksTransactionEntity, LinkedTxn, Address, EmailAddress, QuickbooksBaseObject, CustomerMemo
from .tax import TxnTaxDetail
from .detailline import DetailLine
from ..mixins import DeleteMixin


@python_2_unicode_compatible
class RefundReceiptCheckPayment(QuickbooksBaseObject):
    qbo_object_name = "CheckPayment"

    def __init__(self):
        super(RefundReceiptCheckPayment, self).__init__()
        self.CheckNum = ""
        self.Status = ""
        self.NameOnAcct = ""
        self.AcctNum = ""
        self.BankName = ""

    def __str__(self):
        return self.CheckNum


@python_2_unicode_compatible
class RefundReceipt(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin):
    """
    QBO definition: RefundReceipt represents a refund to the customer for a product or service that was given.
    """
    class_dict = {
        "DepartmentRef": Ref,
        "CurrencyRef": Ref,
        "TxnTaxDetail": TxnTaxDetail,
        "DepositToAccountRef": Ref,
        "CustomerRef": Ref,
        "BillAddr":  Address,
        "ShipAddr":  Address,
        "ClassRef": Ref,
        "BillEmail": EmailAddress,
        "PaymentMethodRef": Ref,
        "CheckPayment": RefundReceiptCheckPayment,
        "CreditCardPayment": CreditCardPayment,
        "CustomerMemo": CustomerMemo,
    }

    list_dict = {
        "CustomField": CustomField,
        "Line": DetailLine,
        "LinkedTxn": LinkedTxn
    }

    detail_dict = {

    }

    qbo_object_name = "RefundReceipt"

    def __init__(self):
        super(RefundReceipt, self).__init__()
        self.DocNumber = ""
        self.TotalAmt = 0
        self.ApplyTaxAfterDiscount = False
        self.PrintStatus = "NotSet"
        self.Balance = 0
        self.PaymentRefNum = ""
        self.TxnDate = ""
        self.ExchangeRate = 1
        self.PrivateNote = ""

        self.PaymentRefNum = ""
        self.PaymentType = ""

        self.TxnSource = None
        self.GlobalTaxCalculation = "TaxExcluded"

        self.DepartmentRef = None
        self.CurrencyRef = None
        self.TxnTaxDetail = None
        self.DepositToAccountRef = None
        self.CustomerRef = None
        self.BillAddr = None
        self.ShipAddr = None
        self.ClassRef = None
        self.BillEmail = None
        self.PaymentMethodRef = None
        self.CheckPayment = None
        self.CreditCardPayment = None
        self.CustomerMemo = None

        self.CustomField = []
        self.Line = []
        self.LinkedTxn = []

    def __str__(self):
        return str(self.TotalAmt)



from datetime import datetime

from quickbooks.objects.account import Account
from quickbooks.objects.bill import Bill
from quickbooks.objects.billpayment import BillPayment, BillPaymentLine, CheckPayment
from quickbooks.objects.vendor import Vendor
from tests.integration.test_base import QuickbooksTestCase


class BillPaymentTest(QuickbooksTestCase):
    def setUp(self):
        super(BillPaymentTest, self).setUp()

        self.account_number = datetime.now().strftime('%d%H%M')
        self.name = "Test Account {0}".format(self.account_number)

    def test_create(self):
        bill_payment = BillPayment()

        bill_payment.PayType = "Check"
        bill_payment.TotalAmt = 200
        bill_payment.PrivateNote = "Private Note"

        vendor = Vendor.all(max_results=1, qb=self.qb_client)[0]
        bill_payment.VendorRef = vendor.to_ref()

        bill_payment.CheckPayment = CheckPayment()
        account = Account.where("AccountSubType = 'Checking'", qb=self.qb_client)[0]
        bill_payment.CheckPayment.BankAccountRef = account.to_ref()

        ap_account = Account.where("AccountSubType = 'AccountsPayable'", qb=self.qb_client)[0]
        bill_payment.APAccountRef = ap_account.to_ref()

        bill = Bill.all(max_results=1, qb=self.qb_client)[0]

        line = BillPaymentLine()
        line.LinkedTxn.append(bill.to_linked_txn())
        line.Amount = 200

        bill_payment.Line.append(line)
        bill_payment.save(qb=self.qb_client)

        query_bill_payment = BillPayment.get(bill_payment.Id, qb=self.qb_client)

        self.assertEquals(query_bill_payment.PayType, "Check")
        self.assertEquals(query_bill_payment.TotalAmt, 200.0)
        self.assertEquals(query_bill_payment.PrivateNote, "Private Note")

        self.assertEquals(len(query_bill_payment.Line), 1)
        self.assertEquals(query_bill_payment.Line[0].Amount, 200.0)



from six import python_2_unicode_compatible
from .base import Address, PhoneNumber, EmailAddress, WebAddress, \
    QuickbooksManagedObject, Ref, MetaData


@python_2_unicode_compatible
class CompanyInfo(QuickbooksManagedObject):
    """
    QBO definition: The CompanyInfo entity contains basic company information.
    In QuickBooks Online, company info and preferences are displayed in the
    same place under preferences, so it may be confusing to figure out from
    user interface which fields may belong to this entity. But in general,
    properties such as company addresses or name are considered company
    information. Some attributes may exist in both CompanyInfo and Preferences
    entities.
    """

    class_dict = {
        "CompanyAddr": Address,
        "CustomerCommunicationAddr": Address,
        "LegalAddr": Address,
        "PrimaryPhone": PhoneNumber,
        "Email": EmailAddress,
        "WebAddr": WebAddress,
        "MetaData": MetaData
    }

    qbo_object_name = "CompanyInfo"

    def __init__(self):
        super(CompanyInfo, self).__init__()

        self.Id = None
        self.CompanyName = ""
        self.LegalName = ""
        self.CompanyStartDate = ""
        self.FiscalYearStartMonth = ""
        self.Country = ""
        self.SupportedLanguages = ""

        self.CompanyAddr = None
        self.CustomerCommunicationAddr = None
        self.LegalAddr = None
        self.PrimaryPhone = None
        self.Email = None
        self.WebAddr = None
        self.MetaData = None

    def __str__(self):
        return self.CompanyName

    def to_ref(self):
        ref = Ref()

        ref.name = self.CompanyName
        ref.type = self.qbo_object_name
        ref.value = self.Id

        return ref



from six import python_2_unicode_compatible
from .base import Ref, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin
from ..mixins import DeleteMixin


@python_2_unicode_compatible
class Transfer(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin):
    """
    QBO definition: A Transfer represents a transaction where funds are moved between two accounts from the
    company's QuickBooks chart of accounts.
    """
    class_dict = {
        "FromAccountRef": Ref,
        "ToAccountRef": Ref,
    }

    qbo_object_name = "Transfer"

    def __init__(self):
        super(Transfer, self).__init__()
        self.Amount = 0
        self.TxnDate = None
        self.PrivateNote = None
        self.TxnSource = None

        self.FromAccountRef = None
        self.ToAccountRef = None

    def __str__(self):
        return str(self.Amount)



from six import python_2_unicode_compatible

from .base import Ref, QuickbooksManagedObject, QuickbooksTransactionEntity, \
    LinkedTxnMixin
from .detailline import DetailLine, AccountBasedExpenseLine, ItemBasedExpenseLine, TDSLine
from ..mixins import DeleteMixin


@python_2_unicode_compatible
class VendorCredit(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin):
    """
    QBO definition: The Vendor Credit entity is an accounts payable transaction that represents a refund or credit
    of payment for goods or services. It is a credit that a vendor owes you for various reasons such as overpaid
    bill, returned merchandise, or other reasons.
    """
    class_dict = {
        "VendorRef": Ref,
        "APAccountRef": Ref,
        "DepartmentRef": Ref,
        "CurrencyRef": Ref,
    }

    list_dict = {
        "Line": DetailLine
    }

    detail_dict = {
        "AccountBasedExpenseLineDetail": AccountBasedExpenseLine,
        "ItemBasedExpenseLineDetail": ItemBasedExpenseLine,
        "TDSLineDetail": TDSLine,
    }

    qbo_object_name = "VendorCredit"

    def __init__(self):
        super(VendorCredit, self).__init__()
        self.DocNumber = ""
        self.TxnDate = ""
        self.PrivateNote = ""
        self.TotalAmt = 0
        self.ExchangeRate = 1
        self.GlobalTaxCalculation = "TaxExcluded"

        self.FromAccountRef = None
        self.ToAccountRef = None

        self.Line = []

    def __str__(self):
        return str(self.TotalAmt)


from six import python_2_unicode_compatible
from .base import Ref, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin, MetaData
from ..mixins import DeleteMixin


@python_2_unicode_compatible
class CreditCardPayment(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin):
    """
    QBO definition: A Represents a financial transaction to record a Credit Card balance payment
    in QuickBooks Online. It provides an easy way for users to move money from a Bank account to
    a Credit Card account. It is essentially a more limited Transfer form.

    Added in QuickBooks Online v1928, Date: February 13, 2020

    https://developer.intuit.com/app/developer/qbo/docs/api/accounting/all-entities/creditcardpayment
    """
    class_dict = {
        "BankAccountRef": Ref,
        "CreditCardAccountRef": Ref,
        "VendorRef": Ref,
        "MetaData": MetaData,
    }

    qbo_object_name = "CreditCardPayment"
    qbo_json_object_name = "CreditCardPaymentTxn"  # JSON object name doesn't match the endpoint name - Thanks Intuit!

    def __init__(self):
        super(CreditCardPayment, self).__init__()
        self.TxnDate = None
        self.Amount = 0
        self.PrivateNote = None
        self.Memo = None
        self.PrintStatus = None
        self.CheckNum = None

        self.BankAccountRef = None
        self.CreditCardAccountRef = None
        self.VendorRef = None
        self.MetaData = None

    def __str__(self):
        return str(self.Amount)



import unittest

from quickbooks.objects.tax import TaxLineDetail, TaxLine, TxnTaxDetail


class TaxLineDetailTests(unittest.TestCase):
    def test_unicode(self):
        detail = TaxLineDetail()
        detail.TaxPercent = 10

        self.assertEquals(str(detail), "10")


class TaxLineTests(unittest.TestCase):
    def test_unicode(self):
        line = TaxLine()
        line.Amount = 100

        self.assertEquals(str(line), "100")


class TxnTaxDetailTests(unittest.TestCase):
    def test_unicode(self):
        detail = TxnTaxDetail()
        detail.TotalTax = 100

        self.assertEquals(str(detail), "100")



from six import python_2_unicode_compatible
from .base import QuickbooksBaseObject, Ref, LinkedTxn, QuickbooksManagedObject, LinkedTxnMixin, \
    QuickbooksTransactionEntity, CustomField, AttachableRef
from ..mixins import DeleteMixin


class CashBackInfo(QuickbooksBaseObject):
    class_dict = {
        "AccountRef": Ref
    }

    def __init__(self):
        super(CashBackInfo, self).__init__()
        self.Amount = 0
        self.Memo = ""
        self.AccountRef = None


class DepositLineDetail(QuickbooksBaseObject):
    class_dict = {
        "Entity": Ref,
        "ClassRef": Ref,
        "AccountRef": Ref,
        "PaymentMethodRef": Ref,
    }

    def __init__(self):
        super(DepositLineDetail, self).__init__()
        self.CheckNum = ""
        self.TxnType = None

        self.Entity = None
        self.ClassRef = None
        self.AccountRef = None
        self.PaymentMethodRef = None


@python_2_unicode_compatible
class DepositLine(QuickbooksBaseObject):
    class_dict = {
        "DepositToAccountRef": Ref,
        "DepositLineDetail": DepositLineDetail,
    }

    list_dict = {
        "LinkedTxn": LinkedTxn,
        "CustomField": CustomField,
    }

    qbo_object_name = "Deposit"

    def __init__(self):
        super(DepositLine, self).__init__()
        self.Id = None
        self.LineNum = 0
        self.Description = ""
        self.Amount = 0
        self.DetailType = "DepositLineDetail"
        self.LinkedTxn = []
        self.CustomField = []

    def __str__(self):
        return str(self.Amount)


@python_2_unicode_compatible
class Deposit(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity, LinkedTxnMixin):
    """
    QBO definition: A deposit object is a transaction that records one or more deposits of the following types:

        -A customer payment, originally held in the Undeposited Funds account, into the Asset Account specified by
        the Deposit.DepositToAccountRef attribute. The Deposit.line.LinkedTxn sub-entity is used in this
        case to hold deposit information.

        -A new direct deposit specified by Deposit.Line.DepositLineDetail line detail.
    """

    class_dict = {
        "DepositToAccountRef": Ref,
        "DepartmentRef": Ref,
        "CurrencyRef": Ref,
        "AttachableRef": AttachableRef,
        "CashBack": CashBackInfo,
    }

    list_dict = {
        "Line": DepositLine
    }

    detail_dict = {
        "DepositLineDetail": DepositLine
    }

    qbo_object_name = "Deposit"

    def __init__(self):
        super(Deposit, self).__init__()
        self.TotalAmt = 0
        self.HomeTotalAmt = 0
        self.TxnDate = ""
        self.DocNumber = ""
        self.ExchangeRate = 1
        self.GlobalTaxCalculation = "TaxExcluded"
        self.PrivateNote = ""
        self.TxnStatus = ""
        self.TxnSource = None

        self.DepositToAccountRef = None
        self.DepartmentRef = None
        self.CurrencyRef = None
        self.AttachableRef = None
        self.Line = []

    def __str__(self):
        return str(self.TotalAmt)



from six import python_2_unicode_compatible
from ..mixins import ToDictMixin, ToJsonMixin, FromJsonMixin, ListMixin, ReadMixin, UpdateMixin


class QuickbooksBaseObject(ToJsonMixin, FromJsonMixin, ToDictMixin):
    class_dict = {}
    list_dict = {}
    detail_dict = {}


class QuickbooksTransactionEntity(QuickbooksBaseObject):
    def __init__(self):
        self.Id = None
        self.SyncToken = 0
        self.sparse = False
        self.domain = "QBO"


class QuickbooksManagedObject(QuickbooksBaseObject, ReadMixin, ListMixin, UpdateMixin):
    pass


class QuickbooksReadOnlyObject(QuickbooksBaseObject, ReadMixin, ListMixin):
    pass


@python_2_unicode_compatible
class MetaData(FromJsonMixin):
    def __init__(self):
        self.CreateTime = ""
        self.LastUpdatedTime = ""

    def __str__(self):
        return "Created {0}".format(self.CreateTime)


class LinkedTxnMixin(object):
    def to_linked_txn(self):
        linked_txn = LinkedTxn()
        linked_txn.TxnId = self.Id
        linked_txn.TxnType = self.qbo_object_name
        linked_txn.TxnLineId = 1

        return linked_txn


@python_2_unicode_compatible
class Address(QuickbooksBaseObject):
    def __init__(self):
        self.Id = None
        self.Line1 = ""
        self.Line2 = ""
        self.Line3 = ""
        self.Line4 = ""
        self.Line5 = ""
        self.City = ""
        self.CountrySubDivisionCode = ""
        self.Country = ""
        self.PostalCode = ""
        self.Lat = ""
        self.Long = ""
        self.Note = ""

    def __str__(self):
        return "{0} {1}, {2} {3}".format(self.Line1, self.City, self.CountrySubDivisionCode, self.PostalCode)


@python_2_unicode_compatible
class PhoneNumber(ToJsonMixin, FromJsonMixin, ToDictMixin):
    def __init__(self):
        self.FreeFormNumber = ""

    def __str__(self):
        return self.FreeFormNumber


@python_2_unicode_compatible
class EmailAddress(QuickbooksBaseObject):
    def __init__(self):
        self.Address = ""

    def __str__(self):
        return self.Address


@python_2_unicode_compatible
class WebAddress(QuickbooksBaseObject):
    def __init__(self):
        self.URI = ""

    def __str__(self):
        return self.URI


@python_2_unicode_compatible
class Ref(QuickbooksBaseObject):
    def __init__(self):
        self.value = ""
        self.name = ""
        self.type = ""

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class CustomField(QuickbooksBaseObject):
    def __init__(self):
        self.DefinitionId = ""
        self.Type = ""
        self.Name = ""
        self.StringValue = ""

    def __str__(self):
        return self.Name


@python_2_unicode_compatible
class LinkedTxn(QuickbooksBaseObject):
    qbo_object_name = "LinkedTxn"

    def __init__(self):
        super(LinkedTxn, self).__init__()
        self.TxnId = 0
        self.TxnType = 0
        self.TxnLineId = 0

    def __str__(self):
        return str(self.TxnId)


@python_2_unicode_compatible
class CustomerMemo(QuickbooksBaseObject):
    def __init__(self):
        super(CustomerMemo, self).__init__()
        self.value = ""

    def __str__(self):
        return self.value


class MarkupInfo(QuickbooksBaseObject):
    class_dict = {
        "PriceLevelRef": Ref,
    }

    def __init__(self):
        super(MarkupInfo, self).__init__()
        self.PercentBased = False
        self.Value = 0
        self.Percent = 0
        self.PriceLevelRef = None


class AttachableRef(QuickbooksBaseObject):
    class_dict = {
        "EntityRef": Ref
    }

    list_dict = {
        "CustomField": CustomField
    }

    qbo_object_name = "AttachableRef"

    def __init__(self):
        super(AttachableRef, self).__init__()

        self.LineInfo = None
        self.IncludeOnSend = False
        self.Inactive = None
        self.NoRefOnly = None

        self.EntityRef = None
        self.CustomField = []



from six import python_2_unicode_compatible

from quickbooks.mixins import ListMixin, ReadMixin
from .base import QuickbooksTransactionEntity, Ref, QuickbooksBaseObject


@python_2_unicode_compatible
class TaxRate(QuickbooksTransactionEntity, QuickbooksBaseObject, ReadMixin, ListMixin):
    """
    QBO definition: A TaxRate object represents rate applied to calculate tax liability. Use the TaxService
    entity to create a taxrate.
    """
    class_dict = {
        "AgencyRef": Ref,
        "TaxReturnLineRef": Ref,
    }

    qbo_object_name = "TaxRate"

    def __init__(self):
        super(TaxRate, self).__init__()

        # All values are readonly - TaxRates cannot be created or modified with the TaxRate object
        self.Name = ""
        self.Description = ""
        self.RateValue = 0
        self.SpecialTaxType = ""
        self.Active = True
        self.DisplayType = ""
        self.EffectiveTaxRate = ""

        self.AgencyRef = None
        self.TaxReturnLineRef = None

    def __str__(self):
        return self.Name





from datetime import datetime

from quickbooks.objects.taxservice import TaxService, TaxRateDetails
from tests.integration.test_base import QuickbooksTestCase


class TaxServiceTest(QuickbooksTestCase):
    def setUp(self):
        super(TaxServiceTest, self).setUp()

        self.name = "TaxCode {0}".format(datetime.now().strftime('%d%H%M'))

    def test_create(self):
        taxservice = TaxService()
        taxservice.TaxCode = self.name
        tax_rate_detail = TaxRateDetails()

        tax_rate_detail.TaxRateName = self.name
        tax_rate_detail.RateValue = 10
        tax_rate_detail.TaxAgencyId = 1
        tax_rate_detail.TaxApplicableOn = "Sales"

        taxservice.TaxRateDetails.append(tax_rate_detail)

        created_taxservice = taxservice.save(qb=self.qb_client)

        self.assertEquals(created_taxservice.TaxCode, self.name)

        detail = created_taxservice.TaxRateDetails[0]
        self.assertEquals(detail.TaxRateName, self.name)
        self.assertEquals(detail.RateValue, 10)
        self.assertEquals(detail.TaxAgencyId, '1')
        self.assertEquals(detail.TaxApplicableOn, "Sales")



import unittest
try:
    from mock import patch
except ImportError:
    from unittest.mock import patch
from quickbooks import batch, client
from quickbooks.objects.customer import Customer
from quickbooks.exceptions import QuickbooksException


class BatchTests(unittest.TestCase):
    def setUp(self):
        self.qb = client.QuickBooks(
            sandbox=False,
            consumer_key="consumer_key",
            consumer_secret="consumer_secret",
            access_token="access_token",
            access_token_secret="access_token_secret",
            company_id="company_id",
            callback_url="callback_url",
            verbose=True
        )

        self.object1 = Customer()
        self.object2 = Customer()
        self.obj_list = [self.object1, self.object2]

    def test_invalid_operation(self):
        self.assertRaises(QuickbooksException, batch.BatchManager, "invalid")

    @patch('quickbooks.batch.BatchManager.process_batch')
    def test_batch_create(self, process_batch):
        batch.batch_create(self.obj_list)
        self.assertTrue(process_batch.called)

    @patch('quickbooks.batch.BatchManager.process_batch')
    def test_batch_update(self, process_batch):
        batch.batch_update(self.obj_list)
        self.assertTrue(process_batch.called)

    @patch('quickbooks.batch.BatchManager.process_batch')
    def test_batch_delete(self, process_batch):
        batch.batch_delete(self.obj_list)
        self.assertTrue(process_batch.called)

    def test_list_to_batch_request(self):
        batch_mgr = batch.BatchManager("create")

        obj_list = [self.object1, self.object2]
        batch_request = batch_mgr.list_to_batch_request(obj_list)

        self.assertEquals(len(batch_request.BatchItemRequest), 2)

        batch_item = batch_request.BatchItemRequest[0]
        self.assertTrue(batch_item.bId)
        self.assertTrue(len(batch_item.bId) < 50)
        self.assertEquals(batch_item.operation, "create")
        self.assertEquals(batch_item.get_object(), self.object1)

    def test_batch_results_to_list(self):
        batch_mgr = batch.BatchManager("create")
        json_data = {"BatchItemResponse": [{"Customer": {"Id": 164}, "bId": "2"},
                                           {"Fault": {"type": "ValidationFault",
                                                      "Error": [{"Message": "Duplicate Name Exists Error",
                                                                 "code": "6240", "Detail": "detail message",
                                                                 "element": ""}]}, "bId": "1"}],
                     "time": "2015-08-10T11:44:02.957-07:00"}

        batch_request = batch_mgr.list_to_batch_request(self.obj_list)
        batch_request.BatchItemRequest[0].bId = "1"
        batch_request.BatchItemRequest[1].bId = "2"

        results = batch_mgr.batch_results_to_list(json_data, batch_request, self.obj_list)

        self.assertEquals(len(results.faults), 1)
        self.assertEquals(len(results.successes), 1)



import unittest
from quickbooks import utils


class UtilsTests(unittest.TestCase):
    def test_build_where_clause(self):
        where_clause = utils.build_where_clause(field1=1,
                                                field2="Someone's Company")

        self.assertTrue("field1 = 1" in where_clause)
        self.assertTrue("field2 = 'Someone\\\'s Company'" in where_clause)

    def test_build_where_clause_unicode(self):
        where_clause = utils.build_where_clause(field1=u"Test 1",
                                                field2=u"Someone's Company")

        self.assertTrue("field1 = 'Test 1'" in where_clause)
        self.assertTrue("field2 = 'Someone\\\'s Company'" in where_clause)

    def test_build_choose_clause_integers(self):
        where_clause = utils.build_choose_clause(choices=[1, 2],
                                                 field="field1")

        self.assertEqual(where_clause, "field1 in (1, 2)")

    def test_build_choose_clause_strings(self):
        where_clause = utils.build_choose_clause(choices=["val1", "val2"],
                                                 field="field1")

        self.assertEqual(where_clause, "field1 in ('val1', 'val2')")

    def test_build_choose_clause_quoted_value(self):
        where_clause = utils.build_choose_clause(choices=["val1",
                                                          "Someone's Company"],
                                                 field="field1")

        self.assertEqual(where_clause, "field1 in ('val1', 'Someone\\\'s Company')")

    def test_build_choose_clause_unicode(self):
        where_clause = utils.build_choose_clause(choices=[u"Test - & % $", u"Another Test"],
                                                 field="field1")

        self.assertEqual(where_clause, "field1 in ('Test - & % $', 'Another Test')")

    def test_build_choose_clause_unicode_escaped(self):
        where_clause = utils.build_choose_clause(choices=[u"Test - & % $", u"Another's Test"],
                                                 field="field1")

        self.assertEqual(where_clause, "field1 in ('Test - & % $', 'Another\\\'s Test')")



from datetime import datetime
from quickbooks.objects.preferences import Preferences
from tests.integration.test_base import QuickbooksTestCase


class PreferencesTest(QuickbooksTestCase):
    def setUp(self):
        super(PreferencesTest, self).setUp()

        self.account_number = datetime.now().strftime('%d%H%M')
        self.name = "Test Account {0}".format(self.account_number)

    def test_get(self):
        preferences = Preferences.get(qb=self.qb_client)

        self.assertEquals(preferences.Id, "1")
        self.assertEquals(preferences.AccountingInfoPrefs.TaxYearMonth, "January")
        self.assertEquals(preferences.ProductAndServicesPrefs.ForPurchase, True)
        self.assertEquals(preferences.VendorAndPurchasesPrefs.BillableExpenseTracking, True)
        self.assertEquals(preferences.TimeTrackingPrefs.WorkWeekStartDate, "Monday")
        self.assertEquals(preferences.OtherPrefs.NameValue[0].Name, "SalesFormsPrefs.DefaultCustomerMessage")

    def test_update(self):
        preferences = Preferences.get(qb=self.qb_client)

        subject = datetime.now().strftime('%d%H%M%S')
        preferences.EmailMessagesPrefs.EstimateMessage.Subject = subject
        preferences.save(qb=self.qb_client)

        preferences_updated = Preferences.get(qb=self.qb_client)
        self.assertEquals(preferences_updated.EmailMessagesPrefs.EstimateMessage.Subject, subject)



import unittest
try:
	from mock import patch
except ImportError:
	from unittest.mock import patch
from quickbooks.cdc import change_data_capture
from quickbooks.objects import Invoice, Customer
from quickbooks import QuickBooks
from datetime import datetime

class ChangeDataCaptureTest(unittest.TestCase):

	def setUp(self):
		self.qb_client = QuickBooks(
			sandbox=True,
			consumer_key="update_consumer_key",
			consumer_secret="update_consumer_secret",
			access_token="update_access_token",
			access_token_secret="update_access_token_secret",
			company_id="update_company_id",
			callback_url="update_callback_url"
		)

		self.cdc_json_response = {
			"CDCResponse": [
				{
					"QueryResponse": [
						{
							"Customer": [
								{
									"Id": 1,
									"DisplayName": "TestCustomer",
									"Job": False,
									"Balance": 0
								}
							],
							"startPosition": 1,
							"maxResults": 1
						},
						{
							"Invoice": [
								{
									"DocNumber": "12344",
									"TxnDate": "2017-01-01",
									"Line": [
										{
											"Id": 1
										},
										{
											"Id": 2
										}
									]
								},
								{
									"DocNumber": "12345",
									"TxnDate": "2017-01-01",
									"Line": [
										{
											"Id": 1
										},
										{
											"Id": 2
										}
									]
								},
							],
							"startPosition": 1,
							"maxResults": 2
						}
					]
				}
			],
			"time": "2016-01-01T00:00:00"
		}

		self.cdc_empty_json_response = {
			"CDCResponse": [
				{
					"QueryResponse": [
						{}
					]
				}
			],
			"time": "2019-03-13T10:24:05.179-07:00"
		}


	@patch('quickbooks.client.QuickBooks.make_request')
	def test_change_data_capture(self, make_request):
		make_request.return_value = self.cdc_json_response.copy()
		cdc_response = change_data_capture([Invoice, Customer], "2017-01-01T00:00:00")
		self.assertEquals(1, len(cdc_response.Customer))
		self.assertEquals(2, len(cdc_response.Invoice))


	@patch('quickbooks.client.QuickBooks.make_request')
	def test_change_data_capture_with_timestamp(self, make_request):
		make_request.return_value = self.cdc_json_response.copy()
		cdc_response_with_datetime = change_data_capture([Invoice, Customer], datetime(2017, 1, 1, 0, 0, 0))
		self.assertEquals(1, len(cdc_response_with_datetime.Customer))
		self.assertEquals(2, len(cdc_response_with_datetime.Invoice))

	@patch('quickbooks.client.QuickBooks.make_request')
	def test_change_data_capture_with_empty_response(self, make_request):
		make_request.return_value = self.cdc_empty_json_response.copy()
		cdc_response = change_data_capture([Invoice, Customer], datetime(2017, 1, 1, 0, 0, 0))

		self.assertFalse(hasattr(cdc_response, 'Customer'))
		self.assertFalse(hasattr(cdc_response, 'Invoice'))



from six import python_2_unicode_compatible
from .base import QuickbooksManagedObject, QuickbooksTransactionEntity, Ref


@python_2_unicode_compatible
class PaymentMethod(QuickbooksManagedObject, QuickbooksTransactionEntity):
    """
    QBO definition: The PaymentMethod entity provides the method of payment for received goods. Delete is achieved by setting the
    Active attribute to false in an entity update request; thus, making it inactive. In this type of delete,
    the record is not permanently deleted, but is hidden for display purposes. References to inactive objects are
    left intact.
    """

    class_dict = {}

    qbo_object_name = "PaymentMethod"

    def __init__(self):
        super(PaymentMethod, self).__init__()
        self.Name = ""
        self.Type = ""
        self.Active = True

    def __str__(self):
        return self.Name

    def to_ref(self):
        ref = Ref()
        ref.name = self.Name
        ref.type = self.qbo_object_name
        ref.value = self.Id

        return ref




import unittest

from quickbooks.objects.budget import BudgetDetail, Budget


class BudgetDetailTests(unittest.TestCase):
    def test_unicode(self):
        budget_detail = BudgetDetail()
        budget_detail.Amount = 10

        self.assertEquals(str(budget_detail), "10")


class BudgetTests(unittest.TestCase):
    def test_unicode(self):
        budget = Budget()
        budget.Name = "test"

        self.assertEquals(str(budget), "test")



from six import python_2_unicode_compatible
from .base import Ref, QuickbooksManagedObject, QuickbooksTransactionEntity, AttachableRef
from ..client import QuickBooks
from ..mixins import DeleteMixin


@python_2_unicode_compatible
class Attachable(DeleteMixin, QuickbooksManagedObject, QuickbooksTransactionEntity):
    """
    QBO definition: This page covers the Attachable, Upload, and Download resources used for attachment management. Attachments are supplemental information linked to a transaction or Item object. They can be files, notes, or a combination of both.
    In the case of file attachments, use an upload endpoint multipart request to upload the files to the QuickBooks attachment list and, optionally, to supply metadata for each via an attachable object. If meta data is not supplied with the upload request, the system creates it.
    In the case of a note, use the create attachable endpoint.
    For information about attachments, see the Attachments Developer Guide.
    """

    class_dict = {
        "EntityRef": Ref,
    }

    list_dict = {
        "AttachableRef": AttachableRef,
    }

    qbo_object_name = "Attachable"

    def __init__(self):
        super(Attachable, self).__init__()

        self.AttachableRef = []
        self.FileName = None
        self._FilePath = ''
        self.Note = ""
        self.FileAccessUri = None
        self.TempDownloadUri = None
        self.Size = None
        self.ContentType = None
        self.Category = None
        self.Lat = None
        self.Long = None
        self.PlaceName = None
        self.ThumbnailFileAccessUri = None
        self.ThumbnailTempDownloadUri = None

    def __str__(self):
        return self.FileName

    def to_ref(self):
        ref = Ref()
        ref.name = self.FileName
        ref.type = self.qbo_object_name
        ref.value = self.Id
        return ref

    def save(self, qb=None):
        if not qb:
            qb = QuickBooks()

        if self.Id and int(self.Id) > 0:
            json_data = qb.update_object(self.qbo_object_name, self.to_json(), _file_path=self._FilePath)
        else:
            json_data = qb.create_object(self.qbo_object_name, self.to_json(), _file_path=self._FilePath)

        if self.FileName:
            obj = type(self).from_json(json_data['AttachableResponse'][0]['Attachable'])
        else:
            obj = type(self).from_json(json_data['Attachable'])

        self.Id = obj.Id

        return obj



import unittest


from quickbooks.exceptions import QuickbooksException, AuthorizationException


class QuickbooksExceptionTests(unittest.TestCase):
    def test_init(self):
        exception = QuickbooksException("message", 100, "detail")

        self.assertEquals(exception.message, "message")
        self.assertEquals(exception.error_code, 100)
        self.assertEquals(exception.detail, "detail")


class AuthorizationExceptionTests(unittest.TestCase):
    def test_unicode(self):
        exception = AuthorizationException("message", detail="detail")

        self.assertEquals(str(exception), "QB Auth Exception 0: message\ndetail")
